@@ -14,16 +14,18 @@
    extern int            number_of_specials;
    extern int            time_used;
    extern int            time_used_opponent;
+   extern int            cpu_time_used;
    extern int            auto_kibitzing;
+   extern int            next_time_check;
    extern int            total_moves;
    extern int            initialized;
    extern int            early_exit;
    extern int            new_game;
-   extern char           auto_kibitz_list[100][20];
-   extern char           GM_list[100][20];
-   extern char           IM_list[100][20];
-   extern char           computer_list[100][20];
-   extern char           special_list[100][20];
+   extern char           auto_kibitz_list[64][20];
+   extern char           GM_list[512][20];
+   extern char           IM_list[512][20];
+   extern char           computer_list[512][20];
+   extern char           special_list[512][20];
    extern FILE           *input_stream;
    extern FILE           *book_file;
    extern FILE           *books_file;
@@ -36,12 +38,22 @@
    extern int            log_id;
    extern int            output_format;
    extern int            EGTBlimit;
-   extern char           whisper_text[500];
+   extern int            EGTB_draw;
+   extern int            EGTB_use;
+   extern char           whisper_text[512];
    extern int            whisper_value;
    extern int            whisper_depth;
    extern int            last_mate_score;
    extern int            last_opponent_move;
 
+   extern int            incheck_depth;
+   extern int            onerep_depth;
+   extern int            pushpp_depth;
+   extern int            recap_depth;
+   extern int            singular_depth;
+   extern int            threat_depth;
+
+   extern int            pgn_suggested_percent;
    extern char           pgn_event[32];
    extern char           pgn_date[32];
    extern char           pgn_round[32];
@@ -58,6 +70,7 @@
    extern int            solutions[10];
    extern int            solution_type;
    extern int            default_draw_score;
+   extern int            accept_draws;
    extern int            over;
    extern int            ics;
    extern int            auto232;
@@ -94,10 +107,6 @@
    extern int            root_total_black_pawns;
    extern int            nodes_per_second;
    extern int            cpu_percent;
-
-   extern int            tb_probes;
-   extern int            tb_probes_successful;
-
    extern int            opening;
    extern int            middle_game;
    extern int            end_game;
@@ -111,7 +120,8 @@
    extern signed char    draw_counter;
    extern signed char    draw_count;
    extern char           audible_alarm;
-   extern char           hint[16];
+   extern char           hint[512];
+   extern char           book_hint[512];
    extern int            post;
    extern int            search_depth;
    extern int            search_move;
@@ -121,13 +131,13 @@
    extern int            absolute_time_limit;
    extern int            search_time_limit;
    extern int            nodes_between_time_checks;
-   extern int            next_time_check;
+   extern int            burp;
 
    extern int            time_abort;
-   extern signed char    pondering;   /* program is thinking on opponent's time */
-   extern signed char    thinking;    /* program is searching on its time       */
-   extern signed char    puzzling;    /* program is puzzling about a move to ponder */
-   extern signed char    booking;     /* program is searching, following book moves */
+   extern signed char    pondering;   /* thinking on opponent's time     */
+   extern signed char    thinking;    /* searching on its time           */
+   extern signed char    puzzling;    /* puzzling about a move to ponder */
+   extern signed char    booking;     /* searching, following book moves */
    extern signed char    abort_search;
    extern int            ponder;
    extern int            ponder_move;
@@ -141,23 +151,15 @@
    extern unsigned int   program_start_time, program_end_time;
    extern unsigned int   start_time, end_time;
    extern unsigned int   elapsed_start, elapsed_end;
-   extern unsigned int   nodes_searched;
-   extern unsigned int   evaluations;
    extern int            predicted;
    extern signed char    transposition_id;
-   extern int            transposition_probes;
-   extern int            transposition_hits;
-   extern int            pawn_probes;
-   extern int            pawn_hits;
-   extern int            check_extensions_done;
-   extern int            recapture_extensions_done;
-   extern int            passed_pawn_extensions_done;
-   extern int            one_reply_extensions_done;
 
    extern int            ansi;
    extern int            trace_level;
+   extern int            max_threads;
+   extern int            min_thread_depth;
+   extern int            thread_depth;
    extern int            display_options;
-   extern int            burp;
    extern unsigned int   noise_level;
 
    extern int            book_move;
@@ -165,6 +167,9 @@
    extern int            book_accept_mask;
    extern int            book_reject_mask;
    extern int            book_random;
+   extern float          book_weight_freq;
+   extern float          book_weight_eval;
+   extern float          book_weight_learn;
    extern int            book_search_trigger;
    extern int            book_selection_width;
    extern int            show_book;
@@ -185,13 +190,12 @@
    extern int            tc_safety_margin;
    extern int            no_tricks;
    extern int            computer_opponent;
+   extern int            draw_score_normal;
    extern int            usage_level;
-
    extern int            log_hash;
    extern int            log_pawn_hash;
    extern int            hash_table_size;
    extern int            pawn_hash_table_size;
-
    extern int            hash_maska;
    extern int            hash_maskb;
    extern unsigned int   pawn_hash_mask;
@@ -201,40 +205,18 @@
    extern HASH_ENTRY      *trans_ref_bb;
    extern PAWN_HASH_ENTRY *pawn_hash_table;
 
-   extern int            hash_move[MAXPLY];
    extern int            history_w[4096], history_b[4096];
-   extern int            killer_move1[MAXPLY];
-   extern int            killer_move2[MAXPLY];
-   extern int            killer_count1[MAXPLY];
-   extern int            killer_count2[MAXPLY];
-   extern BITBOARD       replist_w[50+MAXPLY/2];
-   extern BITBOARD       replist_b[50+MAXPLY/2];
-   extern BITBOARD       *rephead_w;
-   extern BITBOARD       *rephead_b;
 
    extern int            unblocked_pawns[9];
    extern int            p_values[15];
-   extern int            current_move[MAXPLY];
-   extern int            *last[MAXPLY];
-   extern signed char    in_check[MAXPLY];
-   extern signed char    extended_reason[MAXPLY];
-   extern signed char    current_phase[MAXPLY];
-   extern int            move_list[5120];
-   extern int            sort_value[256];
-   extern signed char    searched_this_root_move[256];
-   extern unsigned int   root_nodes[256];
-   extern CHESS_PATH     pv[MAXPLY];
-   extern CHESS_PATH     last_pv;
+   extern PATH     last_pv;
    extern int            last_value;
-   extern NEXT_MOVE      next_status[MAXPLY];
-   extern SEARCH_POSITION position[MAXPLY+2];
-   extern BITBOARD       save_hash_key[MAXPLY+2];
-   extern unsigned int   save_pawn_hash_key[MAXPLY+2];
 
    extern char           white_outpost[64];
    extern char           black_outpost[64];
    extern char           square_color[64];
    extern int            passed_pawn_value[8];
+   extern int            isolated_pawn_value[9];
    extern int            supported_passer[8];
    extern int            reduced_material_passer[20];
    extern int            pawn_advance[8];
@@ -267,13 +249,7 @@
    extern BITBOARD       bishop_attacks[64];
 #if defined(COMPACT_ATTACKS)
   /* Stuff these into a structure to make the addressing slightly cheaper */
-  extern struct at {
-    unsigned char which_attack[8][64];
-    BITBOARD      file_attack_bitboards[8][MAX_ATTACKS_FROM_SQUARE];
-    unsigned char rank_attack_bitboards[8][MAX_ATTACKS_FROM_SQUARE];
-    unsigned char length8_mobility[8][MAX_ATTACKS_FROM_SQUARE];
-    unsigned char short_mobility[NSHORT_MOBILITY];
-  } at;
+  extern struct at at;
 
   extern BITBOARD       diag_attack_bitboards[NDIAG_ATTACKS];
   extern BITBOARD       anti_diag_attack_bitboards[NDIAG_ATTACKS];
@@ -296,8 +272,7 @@
   extern int            rook_mobility_rl90[64][256];
 #endif
 
-   extern CHESS_POSITION search;
-   extern CHESS_POSITION display;
+   extern POSITION display;
 
    extern BITBOARD       queen_attacks[64];
    extern BITBOARD       king_attacks[64];
@@ -345,7 +320,8 @@
    extern BITBOARD       left_side_mask[8];
    extern BITBOARD       right_side_empty_mask[8];
    extern BITBOARD       left_side_empty_mask[8];
-   extern BITBOARD       right_half_mask, left_half_mask;
+   extern BITBOARD       mask_efgh, mask_abcd;
+   extern BITBOARD       mask_fgh, mask_abc;
    extern BITBOARD       mask_abs7_w, mask_abs7_b;
    extern BITBOARD       pawns_cramp_black;
    extern BITBOARD       pawns_cramp_white;
@@ -360,10 +336,6 @@
    extern BITBOARD       mask_B2B3;
    extern BITBOARD       mask_G6G7;
    extern BITBOARD       mask_B6B7;
-   extern BITBOARD       mask_F3H3;
-   extern BITBOARD       mask_F6H6;
-   extern BITBOARD       mask_A3C3;
-   extern BITBOARD       mask_A6C6;
    extern BITBOARD       mask_A7H7;
    extern BITBOARD       mask_A2H2;
    extern BITBOARD       center;
@@ -436,6 +408,7 @@
 
    extern BITBOARD       mask_pawn_protected_b[64];
    extern BITBOARD       mask_pawn_protected_w[64];
+   extern BITBOARD       mask_pawn_duo[64];
    extern BITBOARD       mask_pawn_isolated[64];
    extern BITBOARD       mask_pawn_passed_w[64];
    extern BITBOARD       mask_pawn_passed_b[64];
@@ -460,4 +433,21 @@
    extern BITBOARD       mask_wk_4th, mask_wq_4th, mask_bk_4th, mask_bq_4th;
    extern BITBOARD       mask_wk_5th, mask_wq_5th, mask_bk_5th, mask_bq_5th;
    extern BOOK_POSITION  book_buffer[BOOK_CLUSTER_SIZE];
+   extern BOOK_POSITION  books_buffer[BOOK_CLUSTER_SIZE];
+
+   extern unsigned int   thread_start_time[CPUS];
+#  if defined(SMP)
+   extern TREE           *local[MAX_BLOCKS+1];
+   extern TREE           *volatile thread[CPUS];
+   extern lock_t         lock_hasha, lock_hashb, lock_pawn_hash, lock_smp, lock_io;
+   extern volatile int   smp_idle;
+   extern volatile int   smp_threads;
+   extern pthread_attr_t pthread_attr;
+#  else
+   extern TREE           local_data[1], *local[1];
+#  endif
+   extern unsigned int   parallel_splits;
+   extern unsigned int   parallel_stops;
+   extern unsigned int   max_split_blocks;
+   extern volatile unsigned int   splitting;
 #endif
