@@ -224,45 +224,6 @@ else
 return (color);
 }
 
-/*--> EGMapFromHostPiece: map a piece from the host to the EPD style */
-static
-pT
-EGMapFromHostPiece(siT piece)
-{
-pT p;
-
-/* this is an internal glue routine */
-
-/* map from Crafty's piece representation */
-
-switch (piece)
-	{
-	case pawn:
-		p = p_p;
-		break;
-	case knight:
-		p = p_n;
-		break;
-	case bishop:
-		p = p_b;
-		break;
-	case rook:
-		p = p_r;
-		break;
-	case queen:
-		p = p_q;
-		break;
-	case king:
-		p = p_k;
-		break;
-	default:
-		p = p_nil;
-		break;
-	};
-
-return (p);
-}
-
 /*--> EGMapToHostPiece: map a piece to the host from the EPD style */
 static
 siT
@@ -658,7 +619,7 @@ return (move);
 /*--> EGGetIndexedHostPosition: copy from indexed host position */
 static
 void
-EGGetIndexedHostPosition(siT posdex, int active)
+EGGetIndexedHostPosition(TREE *tree, siT posdex, int active)
 {
 sqT sq;
 rbT rb;
@@ -680,7 +641,7 @@ the previous EPD Kit current position is lost.
 
 for (sq = sq_a1; sq <= sq_h8; sq++)
 	rb.rbv[sq] =
-		EGMapFromHostCP(search.board[EGMapToHostSq(sq)]);
+		EGMapFromHostCP(tree->pos.board[EGMapToHostSq(sq)]);
 
 /* read from the host piece active color */
 
@@ -690,7 +651,7 @@ actc = EGMapFromHostColor((siT)active);
 
 cast = 0;
 
-switch (position[posdex].w_castle)
+switch (tree->position[posdex].w_castle)
 	{
 	case 0:
 		break;
@@ -705,7 +666,7 @@ switch (position[posdex].w_castle)
 		break;
 	};
 
-switch (position[posdex].b_castle)
+switch (tree->position[posdex].b_castle)
 	{
 	case 0:
 		break;
@@ -723,11 +684,11 @@ switch (position[posdex].b_castle)
 /* read from the host piece en passant target square */
 
 epsq = sq_nil;
-if (position[posdex].enpassant_target != 0)
+if (tree->position[posdex].enpassant_target != 0)
 	{
 	sq = sq_a1;
 	while ((epsq == sq_nil) && (sq <= sq_h8))
-		if (position[posdex].enpassant_target == EGMapToHostSq(sq))
+		if (tree->position[posdex].enpassant_target == EGMapToHostSq(sq))
 			epsq = sq;
 		else
 			sq++;
@@ -735,7 +696,7 @@ if (position[posdex].enpassant_target != 0)
 
 /* read from the host halfmove clock */
 
-hmvc = position[posdex].rule_50_moves;
+hmvc = tree->position[posdex].rule_50_moves;
 
 /* read from the host fullmove number */
 
@@ -763,7 +724,7 @@ current position is lost.
 
 /* transfer from ply zero host position */
 
-EGGetIndexedHostPosition(0, wtm);
+EGGetIndexedHostPosition(local[0], 0, wtm);
 
 return;
 }
@@ -781,6 +742,7 @@ sqT epsq;
 siT hmvc;
 siT fmvn;
 siT index;
+TREE *tree=local[0];
 
 /* this is an internal EPD glue routine */
 
@@ -807,7 +769,7 @@ fmvn = EPDFetchFMVN();
 /* copy the board into the host board */
 
 for (sq = sq_a1; sq <= sq_h8; sq++)
-	search.board[EGMapToHostSq(sq)] = EGMapToHostCP(rb.rbv[sq]);
+	tree->pos.board[EGMapToHostSq(sq)] = EGMapToHostCP(rb.rbv[sq]);
 
 /* copy the active color */
 
@@ -815,28 +777,28 @@ wtm = EGMapToHostColor(actc);
 
 /* copy the castling availibility */
 
-position[0].w_castle = 0;
+tree->position[0].w_castle = 0;
 if (cast & cf_wk)
-	position[0].w_castle += 1;
+	tree->position[0].w_castle += 1;
 if (cast & cf_wq)
-	position[0].w_castle += 2;
+	tree->position[0].w_castle += 2;
 
-position[0].b_castle = 0;
+tree->position[0].b_castle = 0;
 if (cast & cf_bk)
-	position[0].b_castle += 1;
+	tree->position[0].b_castle += 1;
 if (cast & cf_bq)
-	position[0].b_castle += 2;
+	tree->position[0].b_castle += 2;
 
 /* copy the en passant target square */
 
 if (epsq == sq_nil)
-	position[0].enpassant_target = 0;
+	tree->position[0].enpassant_target = 0;
 else
-	position[0].enpassant_target = EGMapToHostSq(epsq);
+	tree->position[0].enpassant_target = EGMapToHostSq(epsq);
 
 /* copy the halfmove clock */
 
-position[0].rule_50_moves = hmvc;
+tree->position[0].rule_50_moves = hmvc;
 
 /* copy the fullmove number */
 
@@ -844,10 +806,10 @@ move_number = fmvn;
 
 /* set secondary host data items */
 
-SetChessBitBoards(&position[0]);
+SetChessBitBoards(&tree->position[0]);
 
-rephead_w=replist_w;
-rephead_b=replist_b;
+tree->rephead_w=tree->replist_w;
+tree->rephead_b=tree->replist_b;
 moves_out_of_book = 0;
 last_mate_score = 0;
 
@@ -859,10 +821,8 @@ for (index = 0; index < (sqL * sqL); index++)
 /* clear the host killer information */
 
 for (index = 0; index < MAXPLY; index++) {
-  killer_move1[index] = 0;
-  killer_move2[index] = 0;
-  killer_count1[index]=0;
-  killer_count2[index]=0;
+  tree->killer_move1[index] = 0;
+  tree->killer_move2[index] = 0;
 }
 
 /* clear miscellaneous host items */
@@ -1009,6 +969,7 @@ int move;
 mptrT mptr;
 mT m;
 epdptrT epdptr;
+TREE *tree=local[0];
 
 /* if there is only one move, then select it without search */
 
@@ -1030,10 +991,10 @@ if (EPDFetchMoveCount() == 1)
 
 	/* set Iterate() result variables */
 
-	pv[0].path_hashed = 0;
-	pv[0].path_length = 1;
-	pv[0].path_iteration_depth = 1;
-	pv[0].path[1] = move;
+	tree->pv[0].path_hashed = 0;
+	tree->pv[0].path_length = 1;
+	tree->pv[0].path_iteration_depth = 1;
+	tree->pv[0].path[1] = move;
 	}
 else
 	{
@@ -1052,7 +1013,7 @@ else
 	EPDGenMoves();
 	};
 
-last_pv = pv[0];
+last_pv = tree->pv[0];
 last_value = value;
 
 return (value);
@@ -1073,6 +1034,7 @@ sanT san;
 int move;
 int value;
 char tv[tL];
+TREE *tree=local[0];
 
 /* set flag: no errors (so far) */
 
@@ -1158,11 +1120,11 @@ switch (EPDExtractRefcomIndex(epdptr0))
 		/* execute the supplied move */
 
 		eopptr = EPDLocateEOPCode(epdptr0, epdso_sm);
-		move = InputMove(eopptr->eop_headeov->eov_str, 0, wtm, 0, 0);
+		move = InputMove(tree,eopptr->eop_headeov->eov_str, 0, wtm, 0, 0);
 		fseek(history_file,
 			((((move_number - 1) * 2) + 1 - wtm) * 10), SEEK_SET);
 		fprintf(history_file, "%9s\n", eopptr->eop_headeov->eov_str);
-		MakeMoveRoot(move, wtm);
+		MakeMoveRoot(tree,move, wtm);
 		wtm = ChangeSide(wtm);
 		if (wtm)
 			move_number++;
@@ -1218,11 +1180,11 @@ switch (EPDExtractRefcomIndex(epdptr0))
 		eopptr = EPDLocateEOPCode(epdptr0, epdso_sm);
 		if (eopptr != NULL)
 			{
-			move = InputMove(eopptr->eop_headeov->eov_str, 0, wtm, 0, 0);
+			move = InputMove(tree,eopptr->eop_headeov->eov_str, 0, wtm, 0, 0);
 			fseek(history_file,
 				((((move_number - 1) * 2) + 1 - wtm) * 10), SEEK_SET);
 			fprintf(history_file, "%9s\n", eopptr->eop_headeov->eov_str);
-			MakeMoveRoot(move, wtm);
+			MakeMoveRoot(tree,move, wtm);
 			wtm = ChangeSide(wtm);
 			if (wtm)
 				move_number++;
@@ -1248,7 +1210,7 @@ switch (EPDExtractRefcomIndex(epdptr0))
 			thinking = 1;
 			last_pv.path_iteration_depth = 0;
 			last_pv.path_length = 0;
-			position[1] = position[0];
+			tree->position[1] = tree->position[0];
 
 			/* search */
 
@@ -1256,7 +1218,7 @@ switch (EPDExtractRefcomIndex(epdptr0))
 
 			/* process search result */
 
-			strcpy(tv, OutputMove(&last_pv.path[1], 0, wtm));
+			strcpy(tv, OutputMove(tree,last_pv.path[1], 0, wtm));
 			move = last_pv.path[1];
 
 			/* locate SAN move */
@@ -1273,7 +1235,7 @@ switch (EPDExtractRefcomIndex(epdptr0))
 
 			/* update host position */
 
-			MakeMoveRoot(move, wtm);
+			MakeMoveRoot(tree,move, wtm);
 			wtm = ChangeSide(wtm);
 			if (wtm)
 				move_number++;
@@ -1325,7 +1287,7 @@ switch (EPDExtractRefcomIndex(epdptr0))
 		last_pv.path_iteration_depth = 0;
 		last_pv.path_length = 0;
 
-		InitializeChessBoard(&position[0]);
+		InitializeChessBoard(&tree->position[0]);
 		InitializeHashTables();
 
 		wtm = 1;
@@ -1370,7 +1332,7 @@ return (epdptr1);
 /*--> EGTBScore: fetch a tablebase score for the indicated position */
 nonstatic
 int
-EGTBScore(int posdex, int active, int *scoreptr)
+EGTBScore(TREE *tree, int posdex, int active, int *scoreptr)
 {
 siT flag;
 cpevT cpev;
@@ -1384,7 +1346,7 @@ flag = 0;
 
 /* copy the indexed host position as the EPD Kit new current position */
 
-EGGetIndexedHostPosition((siT)posdex, active);
+EGGetIndexedHostPosition(tree, (siT)posdex, active);
 
 /* perform the probe */
 
@@ -2654,6 +2616,7 @@ eovptrT eovptr;
 eopptrT eopptr;
 epdptrT epdptr;
 char ev[epdL];
+TREE *tree=local[0];
 
 /* this is an internal EPD glue routine */
 
@@ -2764,7 +2727,7 @@ if (flag)
 
 				/* set host search parameters */
 
-				position[1] = position[0];
+				tree->position[1] = tree->position[0];
 				iteration_depth = 0;
 				ponder = 0;
 
@@ -2793,7 +2756,7 @@ if (flag)
 
 				/* extract analysis count: nodes */
 
-				host_acn = nodes_searched;
+				host_acn = tree->nodes_searched;
 				if (host_acn == 0)
 					host_acn = 1;
 
@@ -2828,7 +2791,7 @@ if (flag)
 				EPDDropIfLocEOPCode(epdptr, epdso_pv);
 				eopptr = EPDCreateEOPCode(epdso_pv);
 
-				for (index = 1; index <= (int) pv[0].path_length; index++)
+				for (index = 1; index <= (int) tree->pv[0].path_length; index++)
 					{
 					/* generate moves for the current position */
 
@@ -2836,7 +2799,7 @@ if (flag)
 
 					/* fetch the predicted move at this ply */
 
-					move = pv[0].path[index];
+					move = tree->pv[0].path[index];
 
 					/* map the host move to EPD style */
 
@@ -3766,10 +3729,12 @@ for (tbid = 0; tbid < tbidL; tbid++)
 
 if (count == 0)
 	{
+#ifdef DEBUG
 	sprintf(tbufv,
 		"No tablebase files found in the %s directory.\n",
 		tb_path);
 	EGPrintTB();
+#endif /* DEBUG */
 	}
 else
 	{
