@@ -27,7 +27,7 @@ four columns.  Further testing and development was performed on a
 generic PC running Linux 1.2.9 and using the gcc 2.6.3 compiler.
 */
 
-int EGTBScore(int posdex, int active, int *scoreptr);
+int EGTBScore(TREE *tree, int posdex, int active, int *scoreptr);
 int EGCommandParmCount(char *s);
 int EGCommandCheck(char *s);
 int EGCommand(char *s);
