@@ -31,6 +31,7 @@ void EVTest(char *filename)
   FILE *test_input;
   int i, len;
   char *eof;
+  TREE *tree=local[0];
 /*
  ----------------------------------------------------------
 |                                                          |
@@ -53,7 +54,7 @@ void EVTest(char *filename)
       delim=strchr(buffer,'\n');
       if (delim) *delim=0;
       delim=strchr(buffer,'\r');
-      if (delim) *delim=0;
+      if (delim) *delim=' ';
     }
     else break;
     nargs=ReadParse(buffer,args," ;");
@@ -73,8 +74,8 @@ void EVTest(char *filename)
     }
     else if (!strcmp(args[0],"setboard")) {
       SetBoard(nargs-1,args+1,0);
-      DisplayChessBoard(stdout,search);
-      Print(4095,"Evaluation=%d\n",Evaluate(0,wtm,-999999,999999));
+      DisplayChessBoard(stdout,tree->pos);
+      Print(4095,"Evaluation=%d\n",Evaluate(tree,0,wtm,-999999,999999));
     }
   }
   input_stream=stdin;
