@@ -116,8 +116,7 @@ int diagonal_length[64] = {         1,
                                     1 };
 #endif
 
-void Initialize(int continuing)
-{
+void Initialize(int continuing) {
 /*
  ----------------------------------------------------------
 |                                                          |
@@ -125,22 +124,36 @@ void Initialize(int continuing)
 |                                                          |
  ----------------------------------------------------------
 */
-  int i=0, major, minor;
+  int i, j, major, minor;
+  TREE *tree;
 
+#if defined(SMP)
+  for (i=1;i<MAX_BLOCKS+1;i++) {
+    local[i]=(TREE*) malloc(sizeof(TREE));
+    local[i]->used=0;
+  }
+  local[0]->parent=(TREE*)-1;
+#endif
+  
+  tree=local[0];
+  i=0;
   InitializeZeroMasks();
+#if defined(SMP)
+  InitializeSMP();
+#endif
   InitializeMasks();
   InitializeRandomHash();
   InitializeAttackBoards();
   InitializePawnMasks();
   InitializePieceMasks();
-  InitializeChessBoard(&position[0]);
+  InitializeChessBoard(&tree->position[0]);
 #if defined(NT_i386) || defined(NT_AXP)
   _fmode = _O_BINARY;  /* set global file mode to binary to avoid text translation */
 #endif
 
   EGInit();
 
-  last[0]=move_list;
+  tree->last[0]=tree->move_list;
 
 #if defined(MACOS)
   sprintf(log_filename,":%s:book.bin",book_path);
@@ -151,20 +164,24 @@ void Initialize(int continuing)
   if (!book_file) {
     book_file=fopen(log_filename,"rb");
     if (!book_file) {
-#if defined(MACOS)
+#ifdef DEBUG
+#   if defined(MACOS)
       printf("unable to open book file [:%s:book.bin].\n",book_path);
-#else
+#   else
       printf("unable to open book file [%s/book.bin].\n",book_path);
-#endif
+#   endif
       printf("book is disabled\n");
+#endif /* DEBUG */
     }
     else {
-#if defined(MACOS)
+#ifdef DEBUG
+#   if defined(MACOS)
       printf("unable to open book file [:%s:book.bin] for \"write\".\n",book_path);
-#else
+#   else
       printf("unable to open book file [%s/book.bin] for \"write\".\n",book_path);
-#endif
+#   endif
       printf("learning is disabled\n");
+#endif /* DEBUG */
     }
   }
 #if defined(MACOS)
@@ -183,8 +200,8 @@ void Initialize(int continuing)
     fread(&major,sizeof(int),1,book_file);
     minor=major&65535;
     major=major>>16;
-    if (major<14 || minor<4) {
-      Print(4095,"\nERROR!  book.bin not made by version 14.4 or later\n");
+    if (major<15 || (major==15 && minor<15)) {
+      Print(4095,"\nERROR!  book.bin not made by version 15.15 or later\n");
       book_file=0;
       books_file=0;
     }
@@ -196,13 +213,15 @@ void Initialize(int continuing)
 #endif
   book_lrn_file=fopen(log_filename,"a");
   if (!book_lrn_file) {
-#if defined(MACOS)
+#ifdef DEBUG
+#  if defined(MACOS)
     printf("unable to open book learning file [:%s:book.lrn].\n",book_path);
-#else
+#  else
     printf("unable to open book learning file [%s/book.lrn].\n",book_path);
-#endif
+#  endif
     printf("learning disabled.\n");
-    learning=0;
+#endif /* DEBUG */
+    learning&=~(book_learning+result_learning);
   }
   if (learning&position_learning) {
 #if defined(MACOS)
@@ -211,6 +230,13 @@ void Initialize(int continuing)
     sprintf(log_filename,"%s/position.bin",book_path);
 #endif
     position_file=fopen(log_filename,"rb+");
+    if (position_file) {
+      fseek(position_file,0,SEEK_END);
+      if (ftell(position_file) == 0) {
+        fclose(position_file);
+        position_file=0;
+      }
+    }
     if (!position_file) {
       position_file=fopen(log_filename,"wb+");
       if (position_file) {
@@ -220,13 +246,15 @@ void Initialize(int continuing)
         fwrite(&i,sizeof(int),1,position_file);
       }
       else {
-#if defined(MACOS)
+#ifdef DEBUG
+#   if defined(MACOS)
         printf("unable to open position learning file [:%s:position.bin].\n",book_path);
-#else
+#   else
         printf("unable to open position learning file [%s/position.bin].\n",book_path);
-#endif
+#   endif
         printf("learning disabled.\n");
-        learning=0;
+#endif /* DEBUG */
+        learning&=~position_learning;
       }
     }
 #if defined(MACOS)
@@ -237,7 +265,20 @@ void Initialize(int continuing)
     position_lrn_file=fopen(log_filename,"r");
     if (!position_lrn_file) {
       position_lrn_file=fopen(log_filename,"a");
-      fprintf(position_lrn_file,"position\n");
+      if(position_lrn_file){
+        fprintf(position_lrn_file,"position\n");
+      }
+      else {
+#ifdef DEBUG
+#  if defined(MACOS)
+        printf("unable to open position learning file [:%s:position.lrn].\n",book_path);
+#  else
+        printf("unable to open position learning file [%s/position.lrn].\n",book_path);
+#  endif
+        printf("learning disabled.\n");
+#endif /* DEBUG */
+        learning&=~position_learning;
+      }
     }
     else {
       fclose(position_lrn_file);
@@ -272,13 +313,12 @@ void Initialize(int continuing)
     }
     else {
       sprintf(buffer,"read %s/game.%03d", log_path, log_id);
-      (void) Option();
+      (void) Option(tree);
     }
   }
   else {
     log_file=fopen(log_filename,"w");
     history_file=fopen(history_filename,"w+");
-    fprintf(history_file,"%10s","*");
   }
 
   trans_ref_wa=malloc(16*hash_table_size);
@@ -307,6 +347,16 @@ void Initialize(int continuing)
   hash_maska=(1<<log_hash)-1;
   hash_maskb=(1<<(log_hash+1))-1;
   pawn_hash_mask=((unsigned int) 037777777777)>>(32-log_pawn_hash);
+
+  for (i=0;i<8;i++)
+    for (j=0;j<8;j++) {
+      pawn_value_b[i*8+j]=  pawn_value_w[(7-i)*8+j];
+      knight_value_b[i*8+j]=knight_value_w[(7-i)*8+j];
+      bishop_value_b[i*8+j]=bishop_value_w[(7-i)*8+j];
+      rook_value_b[i*8+j]=  rook_value_w[(7-i)*8+j];
+      queen_value_b[i*8+j]= queen_value_w[(7-i)*8+j];
+      king_value_b[i*8+j]=  king_value_w[(7-i)*8+j];
+    }
 }
 
 void InitializeAttackBoards(void)
@@ -658,6 +708,7 @@ void InitializeAttackBoards(void)
 void InitializeChessBoard(SEARCH_POSITION *new_pos)
 {
   int i;
+  TREE *tree=local[0];
 
   if (strlen(initial_position)) {
     static char a1[80], a2[16], a3[16], a4[16], a5[16];
@@ -668,50 +719,50 @@ void InitializeChessBoard(SEARCH_POSITION *new_pos)
     SetBoard(nargs,args,1);
   }
   else {
-    for(i=0;i<64;i++) search.board[i]=empty;
+    for(i=0;i<64;i++) tree->pos.board[i]=empty;
     new_pos->rule_50_moves=0;
     opening=1;
     middle_game=0;
     end_game=0;
-    largest_positional_score=60;
+    largest_positional_score=100;
 /*
    place pawns
 */
     for (i=0;i<8;i++) {
-      search.board[i+8]=pawn;
-      search.board[i+48]=-pawn;
+      tree->pos.board[i+8]=pawn;
+      tree->pos.board[i+48]=-pawn;
     }
 /*
    place knights
 */
-    search.board[B1]=knight;
-    search.board[G1]=knight;
-    search.board[B8]=-knight;
-    search.board[G8]=-knight;
+    tree->pos.board[B1]=knight;
+    tree->pos.board[G1]=knight;
+    tree->pos.board[B8]=-knight;
+    tree->pos.board[G8]=-knight;
 /*
    place bishops
 */
-    search.board[C1]=bishop;
-    search.board[F1]=bishop;
-    search.board[C8]=-bishop;
-    search.board[F8]=-bishop;
+    tree->pos.board[C1]=bishop;
+    tree->pos.board[F1]=bishop;
+    tree->pos.board[C8]=-bishop;
+    tree->pos.board[F8]=-bishop;
 /*
    place rooks
 */
-    search.board[A1]=rook;
-    search.board[H1]=rook;
-    search.board[A8]=-rook;
-    search.board[H8]=-rook;
+    tree->pos.board[A1]=rook;
+    tree->pos.board[H1]=rook;
+    tree->pos.board[A8]=-rook;
+    tree->pos.board[H8]=-rook;
 /*
    place queens
 */
-    search.board[D1]=queen;
-    search.board[D8]=-queen;
+    tree->pos.board[D1]=queen;
+    tree->pos.board[D8]=-queen;
 /*
    place kings
 */
-    search.board[E1]=king;
-    search.board[E8]=-king;
+    tree->pos.board[E1]=king;
+    tree->pos.board[E8]=-king;
 /*
    initialize castling status so all castling is legal.
 */
@@ -735,184 +786,197 @@ void InitializeChessBoard(SEARCH_POSITION *new_pos)
 void SetChessBitBoards(SEARCH_POSITION *new_pos)
 {
   int i;
-  search.hash_key=0;
-  search.pawn_hash_key=0;
+  TREE *tree=local[0];
+  tree->pos.hash_key=0;
+  tree->pos.pawn_hash_key=0;
 /*
    place pawns
 */
-  search.w_pawn=0;
-  search.b_pawn=0;
+  tree->pos.w_pawn=0;
+  tree->pos.b_pawn=0;
   for (i=0;i<64;i++) {
-    if(search.board[i]==pawn) {
-      search.w_pawn=Or(search.w_pawn,set_mask[i]);
-      search.hash_key=Xor(search.hash_key,w_pawn_random[i]);
-      search.pawn_hash_key=search.pawn_hash_key^w_pawn_random32[i];
+    if(tree->pos.board[i]==pawn) {
+      tree->pos.w_pawn=Or(tree->pos.w_pawn,set_mask[i]);
+      tree->pos.hash_key=Xor(tree->pos.hash_key,w_pawn_random[i]);
+      tree->pos.pawn_hash_key=tree->pos.pawn_hash_key^w_pawn_random32[i];
     }
-    if(search.board[i]==-pawn) {
-      search.b_pawn=Or(search.b_pawn,set_mask[i]);
-      search.hash_key=Xor(search.hash_key,b_pawn_random[i]);
-      search.pawn_hash_key=search.pawn_hash_key^b_pawn_random32[i];
+    if(tree->pos.board[i]==-pawn) {
+      tree->pos.b_pawn=Or(tree->pos.b_pawn,set_mask[i]);
+      tree->pos.hash_key=Xor(tree->pos.hash_key,b_pawn_random[i]);
+      tree->pos.pawn_hash_key=tree->pos.pawn_hash_key^b_pawn_random32[i];
     }
   }
 /*
    place knights
 */
-  search.w_knight=0;
-  search.b_knight=0;
+  tree->pos.w_knight=0;
+  tree->pos.b_knight=0;
   for (i=0;i<64;i++) {
-    if(search.board[i] == knight) {
-      search.w_knight=Or(search.w_knight,set_mask[i]);
-      search.hash_key=Xor(search.hash_key,w_knight_random[i]);
+    if(tree->pos.board[i] == knight) {
+      tree->pos.w_knight=Or(tree->pos.w_knight,set_mask[i]);
+      tree->pos.hash_key=Xor(tree->pos.hash_key,w_knight_random[i]);
     }
-    if(search.board[i] == -knight) {
-      search.b_knight=Or(search.b_knight,set_mask[i]);
-      search.hash_key=Xor(search.hash_key,b_knight_random[i]);
+    if(tree->pos.board[i] == -knight) {
+      tree->pos.b_knight=Or(tree->pos.b_knight,set_mask[i]);
+      tree->pos.hash_key=Xor(tree->pos.hash_key,b_knight_random[i]);
     }
   }
 /*
    place bishops
 */
-  search.w_bishop=0;
-  search.b_bishop=0;
+  tree->pos.w_bishop=0;
+  tree->pos.b_bishop=0;
   for (i=0;i<64;i++) {
-    if(search.board[i] == bishop) {
-      search.w_bishop=Or(search.w_bishop,set_mask[i]);
-      search.hash_key=Xor(search.hash_key,w_bishop_random[i]);
+    if(tree->pos.board[i] == bishop) {
+      tree->pos.w_bishop=Or(tree->pos.w_bishop,set_mask[i]);
+      tree->pos.hash_key=Xor(tree->pos.hash_key,w_bishop_random[i]);
     }
-    if(search.board[i] == -bishop) {
-      search.b_bishop=Or(search.b_bishop,set_mask[i]);
-      search.hash_key=Xor(search.hash_key,b_bishop_random[i]);
+    if(tree->pos.board[i] == -bishop) {
+      tree->pos.b_bishop=Or(tree->pos.b_bishop,set_mask[i]);
+      tree->pos.hash_key=Xor(tree->pos.hash_key,b_bishop_random[i]);
     }
   }
 /*
    place rooks
 */
-  search.w_rook=0;
-  search.b_rook=0;
+  tree->pos.w_rook=0;
+  tree->pos.b_rook=0;
   for (i=0;i<64;i++) {
-    if(search.board[i] == rook) {
-      search.w_rook=Or(search.w_rook,set_mask[i]);
-      search.hash_key=Xor(search.hash_key,w_rook_random[i]);
+    if(tree->pos.board[i] == rook) {
+      tree->pos.w_rook=Or(tree->pos.w_rook,set_mask[i]);
+      tree->pos.hash_key=Xor(tree->pos.hash_key,w_rook_random[i]);
     }
-    if(search.board[i] == -rook) {
-      search.b_rook=Or(search.b_rook,set_mask[i]);
-      search.hash_key=Xor(search.hash_key,b_rook_random[i]);
+    if(tree->pos.board[i] == -rook) {
+      tree->pos.b_rook=Or(tree->pos.b_rook,set_mask[i]);
+      tree->pos.hash_key=Xor(tree->pos.hash_key,b_rook_random[i]);
     }
   }
 /*
    place queens
 */
-  search.w_queen=0;
-  search.b_queen=0;
+  tree->pos.w_queen=0;
+  tree->pos.b_queen=0;
   for (i=0;i<64;i++) {
-    if(search.board[i] == queen) {
-      search.w_queen=Or(search.w_queen,set_mask[i]);
-      search.hash_key=Xor(search.hash_key,w_queen_random[i]);
+    if(tree->pos.board[i] == queen) {
+      tree->pos.w_queen=Or(tree->pos.w_queen,set_mask[i]);
+      tree->pos.hash_key=Xor(tree->pos.hash_key,w_queen_random[i]);
     }
-    if(search.board[i] == -queen) {
-      search.b_queen=Or(search.b_queen,set_mask[i]);
-      search.hash_key=Xor(search.hash_key,b_queen_random[i]);
+    if(tree->pos.board[i] == -queen) {
+      tree->pos.b_queen=Or(tree->pos.b_queen,set_mask[i]);
+      tree->pos.hash_key=Xor(tree->pos.hash_key,b_queen_random[i]);
     }
   }
 /*
    place kings
 */
   for (i=0;i<64;i++) {
-    if(search.board[i] == king) {
-      search.white_king=i;
-      search.hash_key=Xor(search.hash_key,w_king_random[i]);
+    if(tree->pos.board[i] == king) {
+      tree->pos.white_king=i;
+      tree->pos.hash_key=Xor(tree->pos.hash_key,w_king_random[i]);
     }
-    if(search.board[i] == -king) {
-      search.black_king=i;
-      search.hash_key=Xor(search.hash_key,b_king_random[i]);
+    if(tree->pos.board[i] == -king) {
+      tree->pos.black_king=i;
+      tree->pos.hash_key=Xor(tree->pos.hash_key,b_king_random[i]);
     }
   }
   if (new_pos->enpassant_target) 
-    HashEP(new_pos->enpassant_target,search.hash_key);
-  if (!(new_pos->w_castle&1)) HashCastleW(0,search.hash_key);
-  if (!(new_pos->w_castle&2)) HashCastleW(1,search.hash_key);
-  if (!(new_pos->b_castle&1)) HashCastleB(0,search.hash_key);
-  if (!(new_pos->b_castle&2)) HashCastleB(1,search.hash_key);
+    HashEP(new_pos->enpassant_target,tree->pos.hash_key);
+  if (!(new_pos->w_castle&1)) HashCastleW(0,tree->pos.hash_key);
+  if (!(new_pos->w_castle&2)) HashCastleW(1,tree->pos.hash_key);
+  if (!(new_pos->b_castle&1)) HashCastleB(0,tree->pos.hash_key);
+  if (!(new_pos->b_castle&2)) HashCastleB(1,tree->pos.hash_key);
 /*
    initialize combination boards that show multiple pieces.
 */
-  search.bishops_queens=Or(Or(Or(search.w_bishop,search.w_queen),search.b_bishop),search.b_queen);
-  search.rooks_queens=Or(Or(Or(search.w_rook,search.w_queen),search.b_rook),search.b_queen);
-  search.w_occupied=Or(Or(Or(Or(Or(search.w_pawn,search.w_knight),search.w_bishop),search.w_rook),
-                                   search.w_queen),set_mask[search.white_king]);
-  search.b_occupied=Or(Or(Or(Or(Or(search.b_pawn,search.b_knight),search.b_bishop),search.b_rook),
-                                   search.b_queen),set_mask[search.black_king]);
+  tree->pos.bishops_queens=Or(Or(Or(tree->pos.w_bishop,tree->pos.w_queen),tree->pos.b_bishop),tree->pos.b_queen);
+  tree->pos.rooks_queens=Or(Or(Or(tree->pos.w_rook,tree->pos.w_queen),tree->pos.b_rook),tree->pos.b_queen);
+  tree->pos.w_occupied=Or(Or(Or(Or(Or(tree->pos.w_pawn,tree->pos.w_knight),tree->pos.w_bishop),tree->pos.w_rook),
+                                   tree->pos.w_queen),set_mask[tree->pos.white_king]);
+  tree->pos.b_occupied=Or(Or(Or(Or(Or(tree->pos.b_pawn,tree->pos.b_knight),tree->pos.b_bishop),tree->pos.b_rook),
+                                   tree->pos.b_queen),set_mask[tree->pos.black_king]);
 /*
   now initialize rotated occupied bitboards.
 */
-  search.occupied_rl90=0;
-  search.occupied_rl45=0;
-  search.occupied_rr45=0;
+  tree->pos.occupied_rl90=0;
+  tree->pos.occupied_rl45=0;
+  tree->pos.occupied_rr45=0;
   for (i=0;i<64;i++) {
-    if (search.board[i]) {
-      search.occupied_rl90=Or(search.occupied_rl90,set_mask_rl90[i]);
-      search.occupied_rl45=Or(search.occupied_rl45,set_mask_rl45[i]);
-      search.occupied_rr45=Or(search.occupied_rr45,set_mask_rr45[i]);
+    if (tree->pos.board[i]) {
+      tree->pos.occupied_rl90=Or(tree->pos.occupied_rl90,set_mask_rl90[i]);
+      tree->pos.occupied_rl45=Or(tree->pos.occupied_rl45,set_mask_rl45[i]);
+      tree->pos.occupied_rr45=Or(tree->pos.occupied_rr45,set_mask_rr45[i]);
     }
   }
 /*
    initialize black/white piece counts.
 */
-  search.white_pieces=0;
-  search.white_pawns=0;
-  search.black_pieces=0;
-  search.black_pawns=0;
-  search.material_evaluation=0;
+  tree->pos.white_pieces=0;
+  tree->pos.white_majors=0;
+  tree->pos.white_minors=0;
+  tree->pos.white_pawns=0;
+  tree->pos.black_pieces=0;
+  tree->pos.black_majors=0;
+  tree->pos.black_minors=0;
+  tree->pos.black_pawns=0;
+  tree->pos.material_evaluation=0;
   for (i=0;i<64;i++) {
-    switch (search.board[i]) {
+    switch (tree->pos.board[i]) {
       case pawn:
-        search.material_evaluation+=PAWN_VALUE;
-        search.white_pawns+=pawn_v;
+        tree->pos.material_evaluation+=PAWN_VALUE;
+        tree->pos.white_pawns+=pawn_v;
         break;
       case knight:
-        search.material_evaluation+=KNIGHT_VALUE;
-        search.white_pieces+=knight_v;
+        tree->pos.material_evaluation+=KNIGHT_VALUE;
+        tree->pos.white_pieces+=knight_v;
+        tree->pos.white_minors++;
         break;
       case bishop:
-        search.material_evaluation+=BISHOP_VALUE;
-        search.white_pieces+=bishop_v;
+        tree->pos.material_evaluation+=BISHOP_VALUE;
+        tree->pos.white_pieces+=bishop_v;
+        tree->pos.white_minors++;
         break;
       case rook:
-        search.material_evaluation+=ROOK_VALUE;
-        search.white_pieces+=rook_v;
+        tree->pos.material_evaluation+=ROOK_VALUE;
+        tree->pos.white_pieces+=rook_v;
+        tree->pos.white_majors++;
         break;
       case queen:
-        search.material_evaluation+=QUEEN_VALUE;
-        search.white_pieces+=queen_v;
+        tree->pos.material_evaluation+=QUEEN_VALUE;
+        tree->pos.white_pieces+=queen_v;
+        tree->pos.white_majors+=2;
         break;
       case -pawn:
-        search.material_evaluation-=PAWN_VALUE;
-        search.black_pawns+=pawn_v;
+        tree->pos.material_evaluation-=PAWN_VALUE;
+        tree->pos.black_pawns+=pawn_v;
         break;
       case -knight:
-        search.material_evaluation-=KNIGHT_VALUE;
-        search.black_pieces+=knight_v;
+        tree->pos.material_evaluation-=KNIGHT_VALUE;
+        tree->pos.black_pieces+=knight_v;
+        tree->pos.black_minors++;
         break;
       case -bishop:
-        search.material_evaluation-=BISHOP_VALUE;
-        search.black_pieces+=bishop_v;
+        tree->pos.material_evaluation-=BISHOP_VALUE;
+        tree->pos.black_pieces+=bishop_v;
+        tree->pos.black_minors++;
         break;
       case -rook:
-        search.material_evaluation-=ROOK_VALUE;
-        search.black_pieces+=rook_v;
+        tree->pos.material_evaluation-=ROOK_VALUE;
+        tree->pos.black_pieces+=rook_v;
+        tree->pos.black_majors++;
         break;
       case -queen:
-        search.material_evaluation-=QUEEN_VALUE;
-        search.black_pieces+=queen_v;
+        tree->pos.material_evaluation-=QUEEN_VALUE;
+        tree->pos.black_pieces+=queen_v;
+        tree->pos.black_majors+=2;
         break;
       default:
         ;
     }
   }
   TotalPieces=PopCnt(Occupied);
-  if (new_pos == &position[0]) {
-    rephead_b=replist_b;
-    rephead_w=replist_w;
+  if (new_pos == &tree->position[0]) {
+    tree->rephead_b=tree->replist_b;
+    tree->rephead_w=tree->replist_w;
   }
 }
 
@@ -1059,12 +1123,30 @@ void InitializeMasks(void) {
   for (i=1;i<8;i++) file_mask[i]=Shiftr(file_mask[i-1],1);
 /*
   masks to determine if a pawn is protected by another pawn or not.
+  also masks to detect "duos" (pawns side-by-side only).
 */
   for (i=8;i<56;i++) {
-   mask_pawn_protected_w[i]=Or(set_mask[i-1],set_mask[i+1]);
-   if (i > 15) mask_pawn_protected_w[i]|=Or(set_mask[i-7],set_mask[i-9]);
-   mask_pawn_protected_b[i]=Or(set_mask[i-1],set_mask[i+1]);
-   if (i < 48) mask_pawn_protected_b[i]|=Or(set_mask[i+7],set_mask[i+9]);
+    if (File(i)>0 && File(i)<7) {
+      mask_pawn_duo[i]=Or(set_mask[i-1],set_mask[i+1]);
+      mask_pawn_protected_w[i]=Or(set_mask[i-1],set_mask[i+1]);
+      if (i > 15) mask_pawn_protected_w[i]|=Or(set_mask[i-7],set_mask[i-9]);
+      mask_pawn_protected_b[i]=Or(set_mask[i-1],set_mask[i+1]);
+      if (i < 48) mask_pawn_protected_b[i]|=Or(set_mask[i+7],set_mask[i+9]);
+    }
+    else if (File(i) == 0) {
+      mask_pawn_duo[i]=set_mask[i+1];
+      mask_pawn_protected_w[i]=set_mask[i+1];
+      if (i > 15) mask_pawn_protected_w[i]|=set_mask[i-7];
+      mask_pawn_protected_b[i]=set_mask[i+1];
+      if (i < 48) mask_pawn_protected_b[i]|=set_mask[i+9];
+    }
+    else if (File(i) == 7) {
+      mask_pawn_duo[i]=set_mask[i-1];
+      mask_pawn_protected_w[i]=set_mask[i-1];
+      if (i > 15) mask_pawn_protected_w[i]|=set_mask[i-9];
+      mask_pawn_protected_b[i]=set_mask[i-1];
+      if (i < 48) mask_pawn_protected_b[i]|=set_mask[i+7];
+    }
   }
 /*
   masks to select bits on either half of board
@@ -1085,8 +1167,10 @@ void InitializeMasks(void) {
     for (j=i-1;j>=0;j--)
       left_side_empty_mask[i]=Or(left_side_empty_mask[i],file_mask[j]);
   }
-  right_half_mask=Or(Or(Or(file_mask[FILEE],file_mask[FILEF]),file_mask[FILEG]),file_mask[FILEH]);
-  left_half_mask=Or(Or(Or(file_mask[FILEA],file_mask[FILEB]),file_mask[FILEC]),file_mask[FILED]);
+  mask_efgh=Or(Or(Or(file_mask[FILEE],file_mask[FILEF]),file_mask[FILEG]),file_mask[FILEH]);
+  mask_fgh=Or(Or(file_mask[FILEF],file_mask[FILEG]),file_mask[FILEH]);
+  mask_abcd=Or(Or(Or(file_mask[FILEA],file_mask[FILEB]),file_mask[FILEC]),file_mask[FILED]);
+  mask_abc=Or(Or(file_mask[FILEA],file_mask[FILEB]),file_mask[FILEC]);
   mask_kr_trapped_w[0]=set_mask[H2];
   mask_kr_trapped_w[1]=Or(set_mask[H1],set_mask[H2]);
   mask_kr_trapped_w[2]=Or(Or(set_mask[G1],set_mask[H1]),set_mask[H2]);
@@ -1106,10 +1190,6 @@ void InitializeMasks(void) {
   mask_not_rank8=~rank_mask[RANK8];
   mask_not_rank1=~rank_mask[RANK1];
 
-  mask_F3H3=Or(set_mask[F3],set_mask[H3]);
-  mask_F6H6=Or(set_mask[F6],set_mask[H6]);
-  mask_A3C3=Or(set_mask[A3],set_mask[C3]);
-  mask_A6C6=Or(set_mask[A6],set_mask[C6]);
   mask_A7H7=Or(set_mask[A7],set_mask[H7]);
   mask_A2H2=Or(set_mask[A2],set_mask[H2]);
   center=Or(Or(set_mask[D4],set_mask[E4]),
@@ -1330,10 +1410,14 @@ void InitializePawnMasks(void)
   these masks are used to detect that opponent pawns are getting very
   close to the king.
 */
-  mask_wq_4th=Or(Or(set_mask[A4],set_mask[B4]),set_mask[C4]);
-  mask_wq_5th=Or(Or(set_mask[A5],set_mask[B5]),set_mask[C5]);
-  mask_wk_4th=Or(Or(set_mask[F4],set_mask[G4]),set_mask[H4]);
-  mask_wk_5th=Or(Or(set_mask[F5],set_mask[G5]),set_mask[H5]);
+  mask_wq_4th=Or(Or(set_mask[A4],set_mask[B4]),
+                 set_mask[C4]);
+  mask_wq_5th=Or(Or(set_mask[A5],set_mask[B5]),
+                 set_mask[C5]);
+  mask_wk_4th=Or(Or(set_mask[F4],set_mask[G4]),
+                 set_mask[H4]);
+  mask_wk_5th=Or(Or(set_mask[F5],set_mask[G5]),
+                 set_mask[H5]);
   mask_bk_4th=mask_wk_5th;
   mask_bq_4th=mask_wq_5th;
   mask_bk_5th=mask_wk_4th;
@@ -1424,7 +1508,7 @@ void InitializePieceMasks(void)
 *                                                                              *
 *   InitializeRandomHash() is called to initialize the tables of random      *
 *   numbers used to produce the incrementally-updated hash keys.  note that    *
-*   this uses a local random number generator rather than the C library one    *
+*   this uses a treendom number generator rather than the C library one    *
 *   since there is no uniformity in the number of bits returned by the         *
 *   standard library routines, it varies from 16 bits to 64.                   *
 *                                                                              *
@@ -1468,6 +1552,31 @@ void InitializeRandomHash(void)
   }
 }
 
+#if defined(SMP)
+/*
+********************************************************************************
+*                                                                              *
+*   InitlializeSMP() is used to initialize the pthread lock variables.         *
+*                                                                              *
+********************************************************************************
+*/
+void InitializeSMP(void) {
+  int i;
+#if defined(POSIX)
+  pthread_attr_init(&pthread_attr);
+  pthread_attr_setdetachstate(&pthread_attr, PTHREAD_CREATE_DETACHED);
+  pthread_attr_setscope(&pthread_attr, PTHREAD_SCOPE_SYSTEM);
+#endif
+  LockInit(lock_hasha);
+  LockInit(lock_hashb);
+  LockInit(lock_pawn_hash);
+  LockInit(lock_smp);
+  LockInit(lock_io);
+  for (i=0;i<64+1;i++)
+    LockInit(local[i]->lock);
+}
+#endif
+
 void InitializeZeroMasks(void)
 {
   int i,j,maskl,maskr;
