@@ -9,7 +9,7 @@
 #  include <unistd.h>
 #endif
 
-/* last modified 01/08/98 */
+/* last modified 03/11/98 */
 /*
 ********************************************************************************
 *                                                                              *
@@ -30,8 +30,8 @@
 ********************************************************************************
 */
 
-void LearnBook(int wtm, int search_value, int search_depth, int lv, int force)
-{
+void LearnBook(TREE *tree, int wtm, int search_value, int search_depth, int lv,
+               int force) {
 /*
  ----------------------------------------------------------
 |                                                          |
@@ -42,7 +42,9 @@ void LearnBook(int wtm, int search_value, int search_depth, int lv, int force)
 |                                                          |
  ----------------------------------------------------------
 */
+  int t_wtm=wtm;
   if (!book_file) return;
+  if (!(learning&book_learning)) return;
   if (moves_out_of_book <= LEARN_INTERVAL && !force) {
     if (moves_out_of_book) {
       book_learn_eval[moves_out_of_book-1]=search_value;
@@ -60,8 +62,8 @@ void LearnBook(int wtm, int search_value, int search_depth, int lv, int force)
  ----------------------------------------------------------
 */
   else if (moves_out_of_book == LEARN_INTERVAL+1 || force) {
-    int nmoves, move, movenum, i, j, learn_value;
-    int secs, interval;
+    int move, i, j, learn_value, read;
+    int secs, interval, last_book_move=-1;
     float temp_value;
     char cmd[32], buff[80], *nextc;
     int best_eval=-999999, best_eval_p=0;
@@ -72,6 +74,7 @@ void LearnBook(int wtm, int search_value, int search_depth, int lv, int force)
     float book_learn[512], t_learn_value;
 
     if (moves_out_of_book < 1) return;
+    Print(128,"LearnBook() executed\n");
     learning&=~book_learning;
     interval=Min(LEARN_INTERVAL,moves_out_of_book);
     if (interval < 2)  return;
@@ -100,12 +103,14 @@ void LearnBook(int wtm, int search_value, int search_depth, int lv, int force)
     }
     else best_after_worst_eval=book_learn_eval[interval-1];
 
+#if defined(DEBUG)
     Print(128,"Learning analysis ...\n");
     Print(128,"worst=%d  best=%d  baw=%d  wab=%d\n",
           worst_eval, best_eval, best_after_worst_eval, worst_after_best_eval);
     for (i=0;i<interval;i++)
       Print(128,"%d(%d) ",book_learn_eval[i],book_learn_depth[i]);
     Print(128,"\n");
+#endif
 
 /*
  ----------------------------------------------------------
@@ -184,7 +189,7 @@ void LearnBook(int wtm, int search_value, int search_depth, int lv, int force)
  ----------------------------------------------------------
 */
     save_wtm=wtm;
-    InitializeChessBoard(&position[0]);
+    InitializeChessBoard(&tree->position[0]);
     for (i=0;i<512;i++) n_book_moves[i]=0;
     wtm=1;
     for (i=0;i<512;i++) {
@@ -193,31 +198,39 @@ void LearnBook(int wtm, int search_value, int search_depth, int lv, int force)
 
       n_book_moves[i]=0;
       fseek(history_file,i*10,SEEK_SET);
-      fscanf(history_file,"%s",cmd);
-      move=InputMove(cmd,0,wtm,1,0);
-      if (!move) break;
-      position[1]=position[0];
-      last[1]=GenerateCaptures(1, wtm, last[0]);
-      last[1]=GenerateNonCaptures(1, wtm, last[1]);
-      test=HashKey>>49;
-      fseek(book_file,test*sizeof(int),SEEK_SET);
-      fread(&key,sizeof(int),1,book_file);
-      if (key > 0) {
-        fseek(book_file,key,SEEK_SET);
-        fread(&cluster,sizeof(int),1,book_file);
-        fread(book_buffer,sizeof(BOOK_POSITION),cluster,book_file);
+      strcpy(cmd,"");
+      read=fscanf(history_file,"%s",cmd);
+      if (read != 1) break;
+      if (strcmp(cmd,"pass")) {
+        move=InputMove(tree,cmd,0,wtm,1,0);
+        if (!move) break;
+        tree->position[1]=tree->position[0];
+        tree->last[1]=GenerateCaptures(tree, 1, wtm, tree->last[0]);
+        tree->last[1]=GenerateNonCaptures(tree, 1, wtm, tree->last[1]);
+        test=HashKey>>49;
+        fseek(book_file,test*sizeof(int),SEEK_SET);
+        fread(&key,sizeof(int),1,book_file);
+        if (key > 0) {
+          fseek(book_file,key,SEEK_SET);
+          fread(&cluster,sizeof(int),1,book_file);
+          fread(book_buffer,sizeof(BOOK_POSITION),cluster,book_file);
+        }
+        else cluster=0;
+        for (mv=tree->last[0];mv<tree->last[1];mv++) {
+          common=And(HashKey,mask_16);
+          MakeMove(tree, 1,*mv,wtm);
+          temp_hash_key=Xor(HashKey,wtm_random[wtm]);
+          temp_hash_key=Or(And(temp_hash_key,Compl(mask_16)),common);
+          for (j=0;j<cluster;j++)
+            if (!Xor(temp_hash_key,book_buffer[j].position) &&
+                book_buffer[j].learn > (float) LEARN_COUNTER_BAD/100.0) {
+                n_book_moves[i]++;
+                last_book_move=i;
+            }
+          UnMakeMove(tree, 1,*mv,wtm);
+        }
+        if (move) MakeMoveRoot(tree, move,wtm);
       }
-      else cluster=0;
-      for (mv=last[0];mv<last[1];mv++) {
-        common=And(HashKey,mask_16);
-        MakeMove(1,*mv,wtm);
-        temp_hash_key=Xor(HashKey,wtm_random[wtm]);
-        temp_hash_key=Or(And(temp_hash_key,Compl(mask_16)),common);
-        for (j=0;j<cluster;j++)
-          if (!Xor(temp_hash_key,book_buffer[j].position)) n_book_moves[i]++;
-        UnMakeMove(1,*mv,wtm);
-      }
-      if (move) MakeMoveRoot(move,wtm);
       wtm=ChangeSide(wtm);
     } 
 /*
@@ -248,14 +261,18 @@ void LearnBook(int wtm, int search_value, int search_depth, int lv, int force)
 |                                                          |
  ----------------------------------------------------------
 */
-    InitializeChessBoard(&position[0]);
+    InitializeChessBoard(&tree->position[0]);
     wtm=1;
     for (i=0;i<512;i++) {
+      strcpy(cmd,"");
       fseek(history_file,i*10,SEEK_SET);
-      fscanf(history_file,"%s",cmd);
-      move=InputMove(cmd,0,wtm,1,0);
-      if (!move) break;
-      position[1]=position[0];
+      strcpy(cmd,"");
+      read=fscanf(history_file,"%s",cmd);
+      if (read != 1) break;
+      if (strcmp(cmd,"pass")) {
+        move=InputMove(tree,cmd,0,wtm,1,0);
+        if (!move) break;
+        tree->position[1]=tree->position[0];
 /*
  ----------------------------------------------------------
 |                                                          |
@@ -264,10 +281,11 @@ void LearnBook(int wtm, int search_value, int search_depth, int lv, int force)
 |                                                          |
  ----------------------------------------------------------
 */
-      temp_value=book_learn[i];
-      if (wtm != crafty_is_white) temp_value*=-1;
-      LearnBookUpdate(wtm, move, temp_value);
-      MakeMoveRoot(move,wtm);
+        temp_value=book_learn[i];
+        if (wtm != crafty_is_white) temp_value*=-1;
+        LearnBookUpdate(tree, wtm, move, temp_value);
+        MakeMoveRoot(tree, move,wtm);
+      }
       wtm=ChangeSide(wtm);
     } 
 /*
@@ -285,11 +303,12 @@ void LearnBook(int wtm, int search_value, int search_depth, int lv, int force)
     timestruct=localtime((time_t*) &secs);
     fprintf(book_lrn_file,"[Date \"%4d.%02d.%02d\"]\n",timestruct->tm_year+1900,
             timestruct->tm_mon+1,timestruct->tm_mday);
-    nmoves=(move_number-moves_out_of_book)*2-!wtm;
     nextc=buff;
-    for (i=0;i<nmoves;i++) {
+    for (i=0;i<=last_book_move;i++) {
       fseek(history_file,i*10,SEEK_SET);
-      fscanf(history_file,"%s",cmd);
+      strcpy(cmd,"");
+      read=fscanf(history_file,"%s",cmd);
+      if (read!=1) break;
       if (strchr(cmd,' ')) *strchr(cmd,' ')=0;
       sprintf(nextc," %s",cmd);
       nextc=buff+strlen(buff);
@@ -300,34 +319,22 @@ void LearnBook(int wtm, int search_value, int search_depth, int lv, int force)
       }
     }
     fprintf(book_lrn_file,"%s {%d %d %d}\n",
-            buff, (wtm) ? learn_value:-learn_value, search_depth,
+            buff, (t_wtm) ? learn_value:-learn_value, search_depth,
             crafty_rating-opponent_rating);
     fflush(book_lrn_file);
 /*
  ----------------------------------------------------------
 |                                                          |
-|   done.  now use the reset <n> command to reset the      |
-|   position to what it was when we started this process.  |
+|   done.  now restore the game back to where it was       |
+|   before we started all this nonsense.  :)               |
 |                                                          |
  ----------------------------------------------------------
 */
-    nmoves=(move_number-1)*2+save_wtm;
-    wtm=1;
-    movenum=1;
-    InitializeChessBoard(&position[0]);
-    for (i=0;i<nmoves;i++) {
-      fseek(history_file,i*10,SEEK_SET);
-      fscanf(history_file,"%s",cmd);
-      move=InputMove(cmd,0,wtm,1,0);
-      if (move) MakeMoveRoot(move,wtm);
-      wtm=ChangeSide(wtm);
-      if (wtm) movenum++;
-    } 
-    Phase();
+    RestoreGame();
   }
 }
 
-/* last modified 01/08/98 */
+/* last modified 03/11/98 */
 /*
 ********************************************************************************
 *                                                                              *
@@ -340,8 +347,7 @@ void LearnBook(int wtm, int search_value, int search_depth, int lv, int force)
 *                                                                              *
 ********************************************************************************
 */
-void LearnBookUpdate(int wtm, int move, float learn_value)
-{
+void LearnBookUpdate(TREE *tree, int wtm, int move, float learn_value) {
   int cluster, test, move_index, key;
   BITBOARD temp_hash_key, common;
 /*
@@ -362,18 +368,18 @@ void LearnBookUpdate(int wtm, int move, float learn_value)
       fread(&cluster,sizeof(int),1,book_file);
       fread(book_buffer,sizeof(BOOK_POSITION),cluster,book_file);
       common=And(HashKey,mask_16);
-      MakeMove(1,move,wtm);
+      MakeMove(tree, 1,move,wtm);
       temp_hash_key=Xor(HashKey,wtm_random[wtm]);
       temp_hash_key=Or(And(temp_hash_key,Compl(mask_16)),common);
       for (move_index=0;move_index<cluster;move_index++)
         if (!Xor(temp_hash_key,book_buffer[move_index].position)) break;
-      UnMakeMove(1,move,wtm);
+      UnMakeMove(tree, 1,move,wtm);
       if (move_index >= cluster) return;
       if (book_buffer[move_index].learn == 0.0)
         book_buffer[move_index].learn=learn_value;
       else
         book_buffer[move_index].learn=
-          (book_buffer[move_index].learn*4.0+learn_value)/5.0;
+          (book_buffer[move_index].learn+learn_value)/2.0;
       fseek(book_file,key+sizeof(int),SEEK_SET);
       fwrite(book_buffer,sizeof(BOOK_POSITION),cluster,book_file);
       fflush(book_file);
@@ -381,7 +387,7 @@ void LearnBookUpdate(int wtm, int move, float learn_value)
   }
 }
 
-/* last modified 09/15/97 */
+/* last modified 03/11/98 */
 /*
 ********************************************************************************
 *                                                                              *
@@ -394,9 +400,8 @@ void LearnBookUpdate(int wtm, int move, float learn_value)
 *                                                                              *
 ********************************************************************************
 */
-int LearnFunction(int sv, int search_depth,
-                  int rating_difference, int trusted_value)
-{
+int LearnFunction(int sv, int search_depth, int rating_difference,
+                  int trusted_value) {
   float rating_multiplier_trusted[11] = {.00625, .0125, .025, .05, .075, .1,
                                          0.15, 0.2, 0.25, 0.3, 0.35};
   float rating_multiplier_untrusted[11] = {.25, .2, .15, .1, .05, .025, .012,
@@ -414,7 +419,7 @@ int LearnFunction(int sv, int search_depth,
   return(sv*multiplier);
 }
 
-/* last modified 09/15/97 */
+/* last modified 03/11/98 */
 /*
 ********************************************************************************
 *                                                                              *
@@ -439,7 +444,7 @@ int LearnFunction(int sv, int search_depth,
 *                                                                              *
 ********************************************************************************
 */
-void LearnImport(int nargs, char **args) {
+void LearnImport(TREE *tree, int nargs, char **args) {
 
   FILE *learn_in;
   char text[128];
@@ -453,6 +458,7 @@ void LearnImport(int nargs, char **args) {
 |                                                          |
  ----------------------------------------------------------
 */
+  display_options&=4095-128;
   learn_in=fopen(*args,"r");
   if (learn_in == NULL) {
     Print(4095,"unable to open %s for input\n", *args);
@@ -461,11 +467,11 @@ void LearnImport(int nargs, char **args) {
   eof=fscanf(learn_in,"%s",text);
   fclose(learn_in);
   if  (eof == 0) return;
-  if (!strcmp(text,"position")) LearnImportPosition(nargs,args);
-  else LearnImportBook(nargs,args);
+  if (!strcmp(text,"position")) LearnImportPosition(tree,nargs,args);
+  else LearnImportBook(tree,nargs,args);
 }
 
-/* last modified 09/15/97 */
+/* last modified 03/11/98 */
 /*
 ********************************************************************************
 *                                                                              *
@@ -479,11 +485,11 @@ void LearnImport(int nargs, char **args) {
   int index[32768];
 #endif
 
-void LearnImportBook(int nargs, char **args) {
+void LearnImportBook(TREE *tree, int nargs, char **args) {
 
   FILE *learn_in;
   char nextc, text[128], *eof;
-  int wtm, learn_value, depth, rating_difference, move, i, added_lines=0;
+  int wtm, learn_value, depth, rating_difference, move=0, i, added_lines=0;
 
 /*
  ----------------------------------------------------------
@@ -530,9 +536,12 @@ void LearnImportBook(int nargs, char **args) {
  ----------------------------------------------------------
 */
   while (1) {
-    if (added_lines%10==0) printf(".");
+    if (added_lines%10==0) {
+      printf(".");
+      fflush(stdout);
+    }
     if ((added_lines+1)%600 == 0) printf(" (%d)\n",added_lines+1);
-    InitializeChessBoard(&position[0]);
+    InitializeChessBoard(&tree->position[0]);
     wtm=0;
     move_number=0;
     for (i=0;i<100;i++) {
@@ -546,7 +555,7 @@ void LearnImportBook(int nargs, char **args) {
         delim=strchr(text,'\n');
         if (delim) *delim=0;
         delim=strchr(text,'\r');
-        if (delim) *delim=0;
+        if (delim) *delim=' ';
       }
       else break;
       if (strchr(text,'[')) do {
@@ -572,13 +581,13 @@ void LearnImportBook(int nargs, char **args) {
       } while(nextc == ' ' || nextc == '\n');
       if (nextc == '{') break;
       ungetc(nextc,learn_in);
-      move=ReadChessMove(learn_in,wtm,1);
+      move=ReadChessMove(tree,learn_in,wtm,1);
       if (move < 0) break;
-      strcpy(text,OutputMove(&move,0,wtm));
+      strcpy(text,OutputMove(tree,move,0,wtm));
       fseek(history_file,((move_number-1)*2+1-wtm)*10,SEEK_SET);
       fprintf(history_file,"%9s\n",text);
       moves_out_of_book=0;
-      MakeMoveRoot(move,wtm);
+      MakeMoveRoot(tree, move,wtm);
     } while (1);
     if (move < 0) break;
     fscanf(learn_in,"%d %d %d}\n",&learn_value, &depth, &rating_difference);
@@ -587,14 +596,18 @@ void LearnImportBook(int nargs, char **args) {
     for (i=0;i<LEARN_INTERVAL;i++) book_learn_eval[i]=learn_value;
     crafty_rating=rating_difference;
     opponent_rating=0;
-    LearnBook(wtm, (wtm) ? learn_value : -learn_value, depth, 1, 1);
+    learning|=book_learning;
+    if (abs(learn_value) != 99900)
+      LearnBook(tree, wtm, (wtm) ? learn_value : -learn_value, depth, 1, 1);
+    else
+      LearnResult(tree, learn_value < 0);
     added_lines++;
   }
   move_number=1;
   Print(4095,"\nadded %d learned book lines to book.bin\n",added_lines);
 }
 
-/* last modified 09/15/97 */
+/* last modified 03/11/98 */
 /*
 ********************************************************************************
 *                                                                              *
@@ -603,7 +616,7 @@ void LearnImportBook(int nargs, char **args) {
 *                                                                              *
 ********************************************************************************
 */
-void LearnImportPosition(int nargs, char **args) {
+void LearnImportPosition(TREE *tree, int nargs, char **args) {
 
   BITBOARD word1, word2;
   int positions, nextp, secs;
@@ -630,7 +643,7 @@ void LearnImportPosition(int nargs, char **args) {
     delim=strchr(text,'\n');
     if (delim) *delim=0;
     delim=strchr(text,'\r');
-    if (delim) *delim=0;
+    if (delim) *delim=' ';
   }
   if (nargs>1 && !strcmp(args[1],"clear")) {
     fclose(position_file);
@@ -692,7 +705,7 @@ void LearnImportPosition(int nargs, char **args) {
         delim=strchr(text,'\n');
         if (delim) *delim=0;
         delim=strchr(text,'\r');
-        if (delim) *delim=0;
+        if (delim) *delim=' ';
       }
       else break;
       if (strchr(text,'[')) do {
@@ -716,7 +729,7 @@ void LearnImportPosition(int nargs, char **args) {
       delim=strchr(text,'\n');
       if (delim) *delim=0;
       delim=strchr(text,'\r');
-      if (delim) *delim=0;
+      if (delim) *delim=' ';
     }
     nargs=ReadParse(text,args," 	;\n");
     if (strcmp(args[0],"setboard"))
@@ -728,7 +741,7 @@ void LearnImportPosition(int nargs, char **args) {
       delim=strchr(text,'\n');
       if (delim) *delim=0;
       delim=strchr(text,'\r');
-      if (delim) *delim=0;
+      if (delim) *delim=' ';
     }
     else break;
     nargs=ReadParse(text+1,args," 	,;{}\n");
@@ -813,7 +826,7 @@ void LearnImportPosition(int nargs, char **args) {
   fflush(position_lrn_file);
 }
 
-/* last modified 12/05/97 */
+/* last modified 03/11/98 */
 /*
 ********************************************************************************
 *                                                                              *
@@ -847,8 +860,7 @@ void LearnImportPosition(int nargs, char **args) {
 *                                                                              *
 ********************************************************************************
 */
-void LearnPosition(int wtm, int last_value, int value)
-{
+void LearnPosition(TREE *tree, int wtm, int last_value, int value) {
   BITBOARD word1, word2;
   int positions, nextp, secs;
   struct tm *timestruct;
@@ -857,13 +869,6 @@ void LearnPosition(int wtm, int last_value, int value)
   int rank, file, nempty;
 /*
  ----------------------------------------------------------
-|   If position learning files are not accessible: exit    |
- ----------------------------------------------------------
-*/
-if ( (!position_lrn_file) || (!position_file))
-  return;
-/*
- ----------------------------------------------------------
 |                                                          |
 |   is there anything to learn?  if we are already behind  |
 |   a significant amount, losing more is not going to help |
@@ -874,6 +879,7 @@ if ( (!position_lrn_file) || (!position_file))
 |                                                          |
  ----------------------------------------------------------
 */
+  if ((!position_lrn_file) || (!position_file)) return;
   if (last_value < -2*PAWN_VALUE) return;
   if (last_value < value+PAWN_VALUE/3) return;
   if (moves_out_of_book > 10) return;
@@ -888,8 +894,8 @@ if ( (!position_lrn_file) || (!position_file))
   Print(4095,"learning position, wtm=%d  value=%d\n",wtm,value);
   word1=Shiftl((BITBOARD) (value+65536),43);
   word1=Or(word1,Shiftl((BITBOARD) wtm,63));
-  word1=Or(word1,Shiftl((BITBOARD) pv[0].path[1],16));
-  word1=Or(word1,(BITBOARD) (pv[0].path_iteration_depth*INCREMENT_PLY-INCREMENT_PLY));
+  word1=Or(word1,Shiftl((BITBOARD) tree->pv[0].path[1],16));
+  word1=Or(word1,(BITBOARD) (tree->pv[0].path_iteration_depth*INCREMENT_PLY-INCREMENT_PLY));
 
   
   word2=HashKey;
@@ -945,12 +951,12 @@ if ( (!position_lrn_file) || (!position_file))
   if (EnPassant(0)) fprintf(position_lrn_file," %c%c",File(EnPassant(0))+'a',
                             Rank(EnPassant(0))+'1');
   fprintf(position_lrn_file,"\n{%d %d %d}\n",value,
-          pv[0].path[1],
-          pv[0].path_iteration_depth*INCREMENT_PLY);
+          tree->pv[0].path[1],
+          tree->pv[0].path_iteration_depth*INCREMENT_PLY);
   fflush(position_lrn_file);
 }
 
-/* last modified 09/15/97 */
+/* last modified 03/11/98 */
 /*
 ********************************************************************************
 *                                                                              *
@@ -958,8 +964,7 @@ if ( (!position_lrn_file) || (!position_file))
 *                                                                              *
 ********************************************************************************
 */
-void LearnPositionLoad(void)
-{
+void LearnPositionLoad(TREE *tree) {
   register BITBOARD temp_hash_key;
   BITBOARD word1, word2, worda, wordb;
   register HASH_ENTRY *htable;
@@ -1010,3 +1015,161 @@ void LearnPositionLoad(void)
     htable->word2=wordb;
   }
 }
+
+/* last modified 03/11/98 */
+/*
+********************************************************************************
+*                                                                              *
+*   LearnResult() is used to learn from the result of a game that was played   *
+*   to mate or resignation.  Crafty will use the result (if it lost the game)  *
+*   to mark the book move made at the last position where it had more than one *
+*   book move, so that that particular move won't be played again, just as if  *
+*   the learned-value was extremely bad.                                       *
+*                                                                              *
+********************************************************************************
+*/
+
+void LearnResult(TREE *tree, int wtm) {
+  int move, i, j;
+  int secs, last_choice;
+  char cmd[32], buff[80], *nextc;
+  struct tm *timestruct;
+  int n_book_moves[512];
+  int t_wtm=wtm;
+/*
+ ----------------------------------------------------------
+|                                                          |
+|   first, we are going to find every book move in the     |
+|   game, and note how many alternatives there were at     |
+|   every book move.                                       |
+|                                                          |
+ ----------------------------------------------------------
+*/
+  if (!book_file) return;
+  if (!(learning & result_learning)) return;
+  Print(128,"LearnResult() executed\n");
+  learning&=~result_learning;
+  InitializeChessBoard(&tree->position[0]);
+  for (i=0;i<512;i++) n_book_moves[i]=0;
+  wtm=1;
+  for (i=0;i<512;i++) {
+    int *mv, cluster, key, test;
+    BITBOARD common, temp_hash_key;
+
+    n_book_moves[i]=0;
+    strcpy(cmd,"");
+    fseek(history_file,i*10,SEEK_SET);
+    fscanf(history_file,"%s",cmd);
+    if (strcmp(cmd,"pass")) {
+      move=InputMove(tree,cmd,0,wtm,1,0);
+      if (!move) break;
+      tree->position[1]=tree->position[0];
+      tree->last[1]=GenerateCaptures(tree, 1, wtm, tree->last[0]);
+      tree->last[1]=GenerateNonCaptures(tree, 1, wtm, tree->last[1]);
+      test=HashKey>>49;
+      fseek(book_file,test*sizeof(int),SEEK_SET);
+      fread(&key,sizeof(int),1,book_file);
+      if (key > 0) {
+        fseek(book_file,key,SEEK_SET);
+        fread(&cluster,sizeof(int),1,book_file);
+        fread(book_buffer,sizeof(BOOK_POSITION),cluster,book_file);
+      }
+      else cluster=0;
+      for (mv=tree->last[0];mv<tree->last[1];mv++) {
+        common=And(HashKey,mask_16);
+        MakeMove(tree, 1,*mv,wtm);
+        temp_hash_key=Xor(HashKey,wtm_random[wtm]);
+        temp_hash_key=Or(And(temp_hash_key,Compl(mask_16)),common);
+        for (j=0;j<cluster;j++)
+          if (!Xor(temp_hash_key,book_buffer[j].position) &&
+              book_buffer[j].learn > (float) LEARN_COUNTER_BAD/100.0)
+            n_book_moves[i]++;
+        UnMakeMove(tree, 1,*mv,wtm);
+      }
+      if (move) MakeMoveRoot(tree, move,wtm);
+    }
+    wtm=ChangeSide(wtm);
+  } 
+/*
+ ----------------------------------------------------------
+|                                                          |
+|   now, find the last book move where there was more than |
+|   one alternative to choose from.                        |
+|                                                          |
+ ----------------------------------------------------------
+*/
+  if (!t_wtm) {
+    for (last_choice=511;last_choice>=0;last_choice-=2)
+      if (n_book_moves[last_choice] > 1) break;
+  }
+  else {
+    for (last_choice=510;last_choice>=0;last_choice-=2)
+      if (n_book_moves[last_choice] > 1) break;
+  }
+  if (last_choice < 0) return;
+/*
+ ----------------------------------------------------------
+|                                                          |
+|   finally, we run thru the book file and update each     |
+|   book move learned value based on the computation we    |
+|   calculated above.                                      |
+|                                                          |
+ ----------------------------------------------------------
+*/
+  InitializeChessBoard(&tree->position[0]);
+  wtm=1;
+  for (i=0;i<512;i++) {
+    strcpy(cmd,"");
+    fseek(history_file,i*10,SEEK_SET);
+    strcpy(cmd,"");
+    fscanf(history_file,"%s",cmd);
+    if (strcmp(cmd,"pass")) {
+      move=InputMove(tree,cmd,0,wtm,1,0);
+      if (!move) break;
+      tree->position[1]=tree->position[0];
+      if (i == last_choice) LearnBookUpdate(tree, wtm, move, -999.0);
+      MakeMoveRoot(tree, move,wtm);
+    }
+    wtm=ChangeSide(wtm);
+  } 
+/*
+ ----------------------------------------------------------
+|                                                          |
+|   now update the "book.lrn" file so that this can be     |
+|   shared with other crafty users or else saved in case.  |
+|   the book must be re-built.                             |
+|                                                          |
+ ----------------------------------------------------------
+*/
+  fprintf(book_lrn_file,"[White \"%s\"]\n",pgn_white);
+  fprintf(book_lrn_file,"[Black \"%s\"]\n",pgn_black);
+  secs=time(0);
+  timestruct=localtime((time_t*) &secs);
+  fprintf(book_lrn_file,"[Date \"%4d.%02d.%02d\"]\n",timestruct->tm_year+1900,
+          timestruct->tm_mon+1,timestruct->tm_mday);
+  nextc=buff;
+  for (i=0;i<=last_choice;i++) {
+    fseek(history_file,i*10,SEEK_SET);
+    fscanf(history_file,"%s",cmd);
+    if (strchr(cmd,' ')) *strchr(cmd,' ')=0;
+    sprintf(nextc," %s",cmd);
+    nextc=buff+strlen(buff);
+    if (nextc-buff > 60) {
+      fprintf(book_lrn_file,"%s\n",buff);
+      nextc=buff;
+      strcpy(buff,"");
+    }
+  }
+  fprintf(book_lrn_file,"%s {%d %d %d}\n",
+          buff, (t_wtm) ? -99900:99900, 10, 0);
+  fflush(book_lrn_file);
+/*
+ ----------------------------------------------------------
+|                                                          |
+|   done.  now restore the game back to where it was       |
+|   before we started all this nonsense.  :)               |
+|                                                          |
+ ----------------------------------------------------------
+*/
+  RestoreGame();
+}
