@@ -3,7 +3,7 @@
 #include "chess.h"
 #include "data.h"
 
-/* last modified 02/19/97 */
+/* last modified 03/11/98 */
 /*
 ********************************************************************************
 *                                                                              *
@@ -15,10 +15,10 @@
 *                                                                              *
 ********************************************************************************
 */
-static BITBOARD bit_move;
-void MakeMove(int ply, int move, int wtm)
+void MakeMove(TREE *tree, int ply, int move, int wtm)
 {
   register int piece, from, to, captured, promote;
+  BITBOARD bit_move;
 /*
  ----------------------------------------------------------
 |                                                          |
@@ -28,11 +28,11 @@ void MakeMove(int ply, int move, int wtm)
  ----------------------------------------------------------
 */
 #if defined(DEBUG)
-  ValidatePosition(ply,move,"MakeMove(1)");
+  ValidatePosition(tree,ply,move,"MakeMove(1)");
 #endif
-  position[ply+1]=position[ply];
-  save_hash_key[ply]=HashKey;
-  save_pawn_hash_key[ply]=PawnHashKey;
+  tree->position[ply+1]=tree->position[ply];
+  tree->save_hash_key[ply]=HashKey;
+  tree->save_pawn_hash_key[ply]=PawnHashKey;
   if (EnPassant(ply+1)) {
     HashEP(EnPassant(ply+1),HashKey);
     EnPassant(ply+1)=0;
@@ -118,6 +118,7 @@ MakePieceMove:
           HashNW(to,HashKey);
           PieceOnSquare(to)=knight;
           TotalWhitePieces+=knight_v;
+          WhiteMinors++;
           Material+=KNIGHT_VALUE;
           break;
         case bishop:
@@ -126,6 +127,7 @@ MakePieceMove:
           HashBW(to,HashKey);
           PieceOnSquare(to)=bishop;
           TotalWhitePieces+=bishop_v;
+          WhiteMinors++;
           Material+=BISHOP_VALUE;
           break;
         case rook:
@@ -134,6 +136,7 @@ MakePieceMove:
           HashRW(to,HashKey);
           PieceOnSquare(to)=rook;
           TotalWhitePieces+=rook_v;
+          WhiteMajors++;
           Material+=ROOK_VALUE;
           break;
         case queen:
@@ -143,6 +146,7 @@ MakePieceMove:
           HashQW(to,HashKey);
           PieceOnSquare(to)=queen;
           TotalWhitePieces+=queen_v;
+          WhiteMajors+=2;
           Material+=QUEEN_VALUE;
           break;
         }
@@ -198,6 +202,7 @@ MakePieceMove:
           HashNB(to,HashKey);
           PieceOnSquare(to)=-knight;
           TotalBlackPieces+=knight_v;
+          BlackMinors++;
           Material-=KNIGHT_VALUE;
           break;
         case bishop:
@@ -206,6 +211,7 @@ MakePieceMove:
           HashBB(to,HashKey);
           PieceOnSquare(to)=-bishop;
           TotalBlackPieces+=bishop_v;
+          BlackMinors++;
           Material-=BISHOP_VALUE;
           break;
         case rook:
@@ -214,6 +220,7 @@ MakePieceMove:
           HashRB(to,HashKey);
           PieceOnSquare(to)=-rook;
           TotalBlackPieces+=rook_v;
+          BlackMajors++;
           Material-=ROOK_VALUE;
           break;
         case queen:
@@ -223,6 +230,7 @@ MakePieceMove:
           HashQB(to,HashKey);
           PieceOnSquare(to)=-queen;
           TotalBlackPieces+=queen_v;
+          BlackMajors+=2;
           Material-=QUEEN_VALUE;
           break;
         }
@@ -474,6 +482,7 @@ MakePieceMove:
         Clear(to,BlackPieces);
         HashNB(to,HashKey);
         TotalBlackPieces-=knight_v;
+        BlackMinors--;
         Material+=KNIGHT_VALUE;
       }
       else {
@@ -481,6 +490,7 @@ MakePieceMove:
         Clear(to,WhitePieces);
         HashNW(to,HashKey);
         TotalWhitePieces-=knight_v;
+        WhiteMinors--;
         Material-=KNIGHT_VALUE;
       }
     break;
@@ -499,6 +509,7 @@ MakePieceMove:
         Clear(to,BlackPieces);
         HashBB(to,HashKey);
         TotalBlackPieces-=bishop_v;
+        BlackMinors--;
         Material+=BISHOP_VALUE;
       }
       else {
@@ -506,6 +517,7 @@ MakePieceMove:
         Clear(to,WhitePieces);
         HashBW(to,HashKey);
         TotalWhitePieces-=bishop_v;
+        WhiteMinors--;
         Material-=BISHOP_VALUE;
       }
     break;
@@ -534,6 +546,7 @@ MakePieceMove:
           }
         }
         TotalBlackPieces-=rook_v;
+        BlackMajors--;
         Material+=ROOK_VALUE;
       }
       else {
@@ -551,6 +564,7 @@ MakePieceMove:
           }
         }
         TotalWhitePieces-=rook_v;
+        WhiteMajors--;
         Material-=ROOK_VALUE;
       }
     break;
@@ -571,6 +585,7 @@ MakePieceMove:
         Clear(to,BlackPieces);
         HashQB(to,HashKey);
         TotalBlackPieces-=queen_v;
+        BlackMajors-=2;
         Material+=QUEEN_VALUE;
       }
       else {
@@ -578,6 +593,7 @@ MakePieceMove:
         Clear(to,WhitePieces);
         HashQW(to,HashKey);
         TotalWhitePieces-=queen_v;
+        WhiteMajors-=2;
         Material-=QUEEN_VALUE;
       }
       break;
@@ -593,11 +609,11 @@ MakePieceMove:
       Print(128,"piece=%d,from=%d,to=%d,captured=%d\n",
             piece,from,to,captured);
       Print(128,"ply=%d\n",ply);
-      if (log_file) DisplayChessBoard(log_file,search);
+      if (log_file) DisplayChessBoard(log_file,tree->pos);
     }
   }
 #if defined(DEBUG)
-  ValidatePosition(ply+1,move,"MakeMove(2)");
+  ValidatePosition(tree,ply+1,move,"MakeMove(2)");
 #endif
   return;
 }
@@ -613,7 +629,7 @@ MakePieceMove:
 *                                                                              *
 ********************************************************************************
 */
-void MakeMoveRoot(int move, int wtm)
+void MakeMoveRoot(TREE *tree, int move, int wtm)
 {
 /*
  ----------------------------------------------------------
@@ -623,7 +639,7 @@ void MakeMoveRoot(int move, int wtm)
 |                                                          |
  ----------------------------------------------------------
 */
-  MakeMove(0,move,wtm);
+  MakeMove(tree, 0,move,wtm);
 /*
  ----------------------------------------------------------
 |                                                          |
@@ -633,14 +649,14 @@ void MakeMoveRoot(int move, int wtm)
  ----------------------------------------------------------
 */
   if (Rule50Moves(1) == 0) {
-    rephead_b=replist_b;
-    rephead_w=replist_w;
+    tree->rephead_b=tree->replist_b;
+    tree->rephead_w=tree->replist_w;
   }
   WhiteCastle(1)=Max(0,WhiteCastle(1));
   BlackCastle(1)=Max(0,BlackCastle(1));
-  position[0]=position[1];
+  tree->position[0]=tree->position[1];
   if (ChangeSide(wtm))
-    *rephead_w++=HashKey;
+    *tree->rephead_w++=HashKey;
   else
-    *rephead_b++=HashKey;
+    *tree->rephead_b++=HashKey;
 }
