@@ -16,7 +16,7 @@
 *                                                                              *
 ********************************************************************************
 */
-int* GenerateCaptures(int ply, int wtm, int *move)
+int* GenerateCaptures(TREE *tree, int ply, int wtm, int *move)
 {
   register BITBOARD target, piecebd, moves;
   register BITBOARD  promotions, pcapturesl, pcapturesr;
@@ -364,7 +364,7 @@ int* GenerateCaptures(int ply, int wtm, int *move)
 *                                                                              *
 ********************************************************************************
 */
-int* GenerateCheckEvasions(int ply, int wtm, int *move)
+int* GenerateCheckEvasions(TREE *tree, int ply, int wtm, int *move)
 {
   register BITBOARD target, targetc, targetp, piecebd, moves;
   register BITBOARD padvances1, padvances2, pcapturesl, pcapturesr;
@@ -384,15 +384,15 @@ int* GenerateCheckEvasions(int ply, int wtm, int *move)
 */
   if (wtm) {
     king_square=WhiteKingSQ;
-    checksqs=And(AttacksTo(king_square),BlackPieces);
+    checksqs=And(AttacksTo(tree,king_square),BlackPieces);
     checkers=PopCnt(checksqs);
     if (checkers == 1) {
-      checking_square=FirstOne(And(AttacksTo(king_square),
+      checking_square=FirstOne(And(AttacksTo(tree,king_square),
                                     BlackPieces));
       if (PieceOnSquare(checking_square) != -pawn)
         check_direction1=directions[checking_square][king_square];
       target=InterposeSquares(check_direction1,king_square,checking_square);
-      target=Or(target,And(AttacksTo(king_square),BlackPieces));
+      target=Or(target,And(AttacksTo(tree,king_square),BlackPieces));
       target=Or(target,BlackKing);
     }
     else {
@@ -430,7 +430,7 @@ int* GenerateCheckEvasions(int ply, int wtm, int *move)
     temp=from+(king<<12);
     while (moves) {
       to=LastOne(moves);
-      if (!Attacked(to,0) && (directions[from][to] != check_direction1) &&
+      if (!Attacked(tree,to,0) && (directions[from][to] != check_direction1) &&
                                  (directions[from][to] != check_direction2))
         *move++=temp|(to<<6)|((-PieceOnSquare(to))<<15);
       Clear(to,moves);
@@ -449,7 +449,7 @@ int* GenerateCheckEvasions(int ply, int wtm, int *move)
       piecebd=WhiteKnights;
       while (piecebd) {
         from=LastOne(piecebd);
-        if (!PinnedOnKing(wtm,from)) {
+        if (!PinnedOnKing(tree,wtm,from)) {
           moves=And(knight_attacks[from],target);
           temp=from+(knight<<12);
           while (moves) {
@@ -473,7 +473,7 @@ int* GenerateCheckEvasions(int ply, int wtm, int *move)
       piecebd=WhiteBishops;
       while (piecebd) {
         from=LastOne(piecebd);
-        if (!PinnedOnKing(wtm,from)) {
+        if (!PinnedOnKing(tree,wtm,from)) {
           moves=And(AttacksBishop(from),target);
           temp=from+(bishop<<12);
           while (moves) {
@@ -497,7 +497,7 @@ int* GenerateCheckEvasions(int ply, int wtm, int *move)
       piecebd=WhiteRooks;
       while (piecebd) {
         from=LastOne(piecebd);
-        if (!PinnedOnKing(wtm,from)) {
+        if (!PinnedOnKing(tree,wtm,from)) {
           moves=And(AttacksRook(from),target);
           temp=from+(rook<<12);
           while (moves) {
@@ -521,7 +521,7 @@ int* GenerateCheckEvasions(int ply, int wtm, int *move)
       piecebd=WhiteQueens;
       while (piecebd) {
         from=LastOne(piecebd);
-        if (!PinnedOnKing(wtm,from)) {
+        if (!PinnedOnKing(tree,wtm,from)) {
           moves=And(AttacksQueen(from),target);
           temp=from+(queen<<12);
           while (moves) {
@@ -562,12 +562,12 @@ int* GenerateCheckEvasions(int ply, int wtm, int *move)
 */
       while (padvances2) {
         to=LastOne(padvances2);
-        if (!PinnedOnKing(wtm,to-16)) *move++=(to-16)|(to<<6)|(pawn<<12);
+        if (!PinnedOnKing(tree,wtm,to-16)) *move++=(to-16)|(to<<6)|(pawn<<12);
         Clear(to,padvances2);
       }
       while (padvances1) {
         to=LastOne(padvances1);
-        if (!PinnedOnKing(wtm,to-8)) {
+        if (!PinnedOnKing(tree,wtm,to-8)) {
           if (to < 56) *move++=(to-8)|(to<<6)|(pawn<<12);
           else {
             *move++=(to-8)|(to<<6)|(pawn<<12)|(queen<<18);
@@ -587,7 +587,7 @@ int* GenerateCheckEvasions(int ply, int wtm, int *move)
       pcapturesr=And(Shiftr(And(WhitePawns,mask_right_edge),9),targetc);
       while (pcapturesl) {
         to=LastOne(pcapturesl);
-        if (!PinnedOnKing(wtm,to-7)) {
+        if (!PinnedOnKing(tree,wtm,to-7)) {
           if (to < 56) {
             if(PieceOnSquare(to)) 
               *move++=(to-7)|(to<<6)|(pawn<<12)|((-PieceOnSquare(to))<<15);
@@ -605,7 +605,7 @@ int* GenerateCheckEvasions(int ply, int wtm, int *move)
       }
       while (pcapturesr) {
         to=LastOne(pcapturesr);
-        if (!PinnedOnKing(wtm,to-9)) {
+        if (!PinnedOnKing(tree,wtm,to-9)) {
           if (to < 56) {
             if(PieceOnSquare(to)) 
               *move++=(to-9)|(to<<6)|(pawn<<12)|((-PieceOnSquare(to))<<15);
@@ -634,15 +634,15 @@ int* GenerateCheckEvasions(int ply, int wtm, int *move)
 */
   else {
     king_square=BlackKingSQ;
-    checksqs=And(AttacksTo(king_square),WhitePieces);
+    checksqs=And(AttacksTo(tree,king_square),WhitePieces);
     checkers=PopCnt(checksqs);
     if (checkers == 1) {
-      checking_square=FirstOne(And(AttacksTo(king_square),
+      checking_square=FirstOne(And(AttacksTo(tree,king_square),
                                     WhitePieces));
       if (PieceOnSquare(checking_square) != pawn)
         check_direction1=directions[checking_square][king_square];
       target=InterposeSquares(check_direction1,king_square,checking_square);
-      target=Or(target,And(AttacksTo(king_square),WhitePieces));
+      target=Or(target,And(AttacksTo(tree,king_square),WhitePieces));
       target=Or(target,WhiteKing);
     }
     else {
@@ -680,7 +680,7 @@ int* GenerateCheckEvasions(int ply, int wtm, int *move)
     temp=from+(king<<12);
     while (moves) {
       to=FirstOne(moves);
-      if (!Attacked(to,1) && (directions[from][to] != check_direction1) &&
+      if (!Attacked(tree,to,1) && (directions[from][to] != check_direction1) &&
                                  (directions[from][to] != check_direction2))
         *move++=temp|(to<<6)|(PieceOnSquare(to)<<15);
       Clear(to,moves);
@@ -699,7 +699,7 @@ int* GenerateCheckEvasions(int ply, int wtm, int *move)
       piecebd=BlackKnights;
       while (piecebd) {
         from=FirstOne(piecebd);
-        if (!PinnedOnKing(wtm,from)) {
+        if (!PinnedOnKing(tree,wtm,from)) {
           moves=And(knight_attacks[from],target);
           temp=from+(knight<<12);
           while (moves) {
@@ -723,7 +723,7 @@ int* GenerateCheckEvasions(int ply, int wtm, int *move)
       piecebd=BlackBishops;
       while (piecebd) {
         from=FirstOne(piecebd);
-        if (!PinnedOnKing(wtm,from)) {
+        if (!PinnedOnKing(tree,wtm,from)) {
           moves=And(AttacksBishop(from),target);
           temp=from+(bishop<<12);
           while (moves) {
@@ -747,7 +747,7 @@ int* GenerateCheckEvasions(int ply, int wtm, int *move)
       piecebd=BlackRooks;
       while (piecebd) {
         from=FirstOne(piecebd);
-        if (!PinnedOnKing(wtm,from)) {
+        if (!PinnedOnKing(tree,wtm,from)) {
           moves=And(AttacksRook(from),target);
           temp=from+(rook<<12);
           while (moves) {
@@ -769,7 +769,7 @@ int* GenerateCheckEvasions(int ply, int wtm, int *move)
       piecebd=BlackQueens;
       while (piecebd) {
         from=FirstOne(piecebd);
-        if (!PinnedOnKing(wtm,from)) {
+        if (!PinnedOnKing(tree,wtm,from)) {
           moves=And(AttacksQueen(from),target);
           temp=from+(queen<<12);
           while (moves) {
@@ -816,12 +816,12 @@ int* GenerateCheckEvasions(int ply, int wtm, int *move)
 */
       while (padvances2) {
         to=FirstOne(padvances2);
-        if (!PinnedOnKing(wtm,to+16)) *move++=(to+16)|(to<<6)|(pawn<<12);
+        if (!PinnedOnKing(tree,wtm,to+16)) *move++=(to+16)|(to<<6)|(pawn<<12);
         Clear(to,padvances2);
       }
       while (padvances1) {
         to=FirstOne(padvances1);
-        if (!PinnedOnKing(wtm,to+8)) {
+        if (!PinnedOnKing(tree,wtm,to+8)) {
           if (to > 7) *move++=(to+8)|(to<<6)|(pawn<<12);
           else {
             *move++=(to+8)|(to<<6)|(pawn<<12)|(queen<<18);
@@ -841,7 +841,7 @@ int* GenerateCheckEvasions(int ply, int wtm, int *move)
       pcapturesr=And(Shiftl(And(BlackPawns,mask_right_edge),7),targetc);
       while (pcapturesl) {
         to=FirstOne(pcapturesl);
-        if (!PinnedOnKing(wtm,to+9)) {
+        if (!PinnedOnKing(tree,wtm,to+9)) {
           if (to > 7) {
             if(PieceOnSquare(to)) 
               *move++=(to+9)|(to<<6)|(pawn<<12)|(PieceOnSquare(to)<<15);
@@ -859,7 +859,7 @@ int* GenerateCheckEvasions(int ply, int wtm, int *move)
       }
       while (pcapturesr) {
         to=FirstOne(pcapturesr);
-        if (!PinnedOnKing(wtm,to+7)) {
+        if (!PinnedOnKing(tree,wtm,to+7)) {
           if (to > 7) {
             if(PieceOnSquare(to)) 
               *move++=(to+7)|(to<<6)|(pawn<<12)|(PieceOnSquare(to)<<15);
@@ -903,7 +903,7 @@ int* GenerateCheckEvasions(int ply, int wtm, int *move)
 *                                                                              *
 ********************************************************************************
 */
-int* GenerateNonCaptures(int ply, int wtm, int *move)
+int* GenerateNonCaptures(TREE *tree, int ply, int wtm, int *move)
 {
   register BITBOARD target, piecebd, moves;
   register BITBOARD  padvances1, padvances2, pcapturesl, pcapturesr;
@@ -919,11 +919,11 @@ int* GenerateNonCaptures(int ply, int wtm, int *move)
 */
     if (WhiteCastle(ply) > 0) {
       if ((WhiteCastle(ply)&1) && !And(Occupied,Shiftr(mask_2,5)) &&
-          !Attacked(E1,0) && !Attacked(F1,0) && !Attacked(G1,0)) {
+          !Attacked(tree,E1,0) && !Attacked(tree,F1,0) && !Attacked(tree,G1,0)) {
         *move++=12676;
       }
       if ((WhiteCastle(ply)&2) && !And(Occupied,Shiftr(mask_3,1)) &&
-          !Attacked(C1,0) && !Attacked(D1,0) && !Attacked(E1,0)) {
+          !Attacked(tree,C1,0) && !Attacked(tree,D1,0) && !Attacked(tree,E1,0)) {
         *move++=12420;
       }
     }
@@ -1109,11 +1109,11 @@ int* GenerateNonCaptures(int ply, int wtm, int *move)
   else {
     if (BlackCastle(ply) > 0) {
       if ((BlackCastle(ply)&1) && !And(Occupied,Shiftr(mask_2,61)) &&
-          !Attacked(E8,1) && !Attacked(F8,1) && !Attacked(G8,1)) {
+          !Attacked(tree,E8,1) && !Attacked(tree,F8,1) && !Attacked(tree,G8,1)) {
         *move++=16316;
       }
       if ((BlackCastle(ply)&2) && !And(Occupied,Shiftr(mask_3,57)) &&
-          !Attacked(C8,1) && !Attacked(D8,1) && !Attacked(E8,1)) {
+          !Attacked(tree,C8,1) && !Attacked(tree,D8,1) && !Attacked(tree,E8,1)) {
         *move++=16060;
       }
     }
