@@ -5,7 +5,7 @@
 #include "data.h"
 #include "epdglue.h"
 
-/* modified 01/07/98 */
+/* last modified 06/05/98 */
 /*
 ********************************************************************************
 *                                                                              *
@@ -18,11 +18,11 @@
 *                                                                              *
 ********************************************************************************
 */
-int Search(int alpha, int beta, int wtm, int depth, int ply, int do_null)
-{
-  register int first_move=1;
+int Search(TREE *tree, int alpha, int beta, int wtm, int depth,
+           int ply, int do_null) {
+  register int moves_searched=0;
   register BITBOARD save_hash_key;
-  register int initial_alpha, value;
+  register int initial_alpha, value=0;
   register int extensions, pieces;
   int threat=0;
 /*
@@ -38,8 +38,7 @@ int Search(int alpha, int beta, int wtm, int depth, int ply, int do_null)
 |                                                          |
  ----------------------------------------------------------
 */
-  if (ply >= MAXPLY-1) return(beta);
-  nodes_searched++;
+  tree->nodes_searched++;
   if (--next_time_check <= 0) {
     next_time_check=nodes_between_time_checks;
     if (CheckInput()) Interrupt(ply);
@@ -49,6 +48,7 @@ int Search(int alpha, int beta, int wtm, int depth, int ply, int do_null)
       return(0);
     }
   }
+  if (ply >= MAXPLY-1) return(beta);
 /*
  ----------------------------------------------------------
 |                                                          |
@@ -56,9 +56,9 @@ int Search(int alpha, int beta, int wtm, int depth, int ply, int do_null)
 |                                                          |
  ----------------------------------------------------------
 */
-  if (RepetitionCheck(ply,wtm)) {
-    value=(wtm==root_wtm) ? DrawScore() : -DrawScore();
-    if (value < beta) SavePV(ply,value,0);
+  if (RepetitionCheck(tree,ply,wtm)) {
+    value=DrawScore(root_wtm==wtm);
+    if (value < beta) SavePV(tree,ply,value,0);
 #if !defined(FAST)
     if(ply <= trace_level) printf("draw by repetition detected, ply=%d.\n",ply);
 #endif
@@ -97,9 +97,9 @@ int Search(int alpha, int beta, int wtm, int depth, int ply, int do_null)
 |                                                          |
  ----------------------------------------------------------
 */
-  switch (LookUp(ply,depth,wtm,&alpha,&beta,&threat)) {
+  switch (LookUp(tree,ply,depth,wtm,&alpha,&beta,&threat)) {
     case EXACT_SCORE:
-      if(alpha < beta) SavePV(ply,alpha,1);
+      if(alpha < beta) SavePV(tree,ply,alpha,1);
       return(alpha);
     case LOWER_BOUND:
       return(beta);
@@ -120,8 +120,8 @@ int Search(int alpha, int beta, int wtm, int depth, int ply, int do_null)
 |                                                          |
  ----------------------------------------------------------
 */
-  if (EGTBlimit) {
-    if (TotalPieces<=EGTBlimit &&
+  if (EGTB_use) {
+    if (TotalPieces<=EGTB_use &&
         (TotalPieces<5 || (WhiteRooks && BlackRooks))) do {
       register int wpawn, bpawn;
       int tb_value;
@@ -134,15 +134,24 @@ int Search(int alpha, int beta, int wtm, int depth, int ply, int do_null)
              EnPassant(ply)) break;
         }
       }
-      tb_probes++;
-      if (EGTBScore(ply, wtm, &tb_value)) {
-        tb_probes_successful++;
+      tree->tb_probes++;
+#if defined(SMP)
+      Lock(lock_io);
+#endif
+      if (EGTBScore(tree, ply, wtm, &tb_value)) {
+#if defined(SMP)
+        UnLock(lock_io);
+#endif
+        tree->tb_probes_successful++;
         alpha=tb_value;
         if (abs(alpha) > MATE-100) alpha+=(alpha > 0) ? -(ply-1) : +(ply-1);
-        else if (alpha == 0) alpha=(wtm==root_wtm) ? DrawScore() : -DrawScore();
-        if(alpha < beta) SavePV(ply,alpha,2);
+        else if (alpha == 0) alpha=DrawScore(root_wtm==wtm);
+        if(alpha < beta) SavePV(tree,ply,alpha,2);
         return(alpha);
       }
+#if defined(SMP)
+      UnLock(lock_io);
+#endif
     } while(0);
   }
 /*
@@ -152,12 +161,10 @@ int Search(int alpha, int beta, int wtm, int depth, int ply, int do_null)
 |                                                          |
  ----------------------------------------------------------
 */
-  in_check[ply+1]=0;
-  extended_reason[ply+1]=no_extension;
+  tree->in_check[ply+1]=0;
+  tree->extended_reason[ply+1]=no_extension;
   initial_alpha=alpha;
-  last[ply]=last[ply-1];
-  killer_count1[ply+1]=0;
-  killer_count2[ply+1]=0;
+  tree->last[ply]=tree->last[ply-1];
 /*
  ----------------------------------------------------------
 |                                                          |
@@ -199,27 +206,27 @@ int Search(int alpha, int beta, int wtm, int depth, int ply, int do_null)
 */
 # if defined(NULL_MOVE_DEPTH)
   pieces=(wtm) ? TotalWhitePieces : TotalBlackPieces;
-  if (do_null && !in_check[ply] && pieces &&
+  if (do_null && !tree->in_check[ply] && pieces &&
       (pieces>5 || depth<421)) {
-    current_move[ply]=0;
-    current_phase[ply]=NULL_MOVE;
+    tree->current_move[ply]=0;
+    tree->current_phase[ply]=NULL_MOVE;
 #if !defined(FAST)
     if (ply <= trace_level)
-      SearchTrace(ply,depth,wtm,alpha,beta,"Search",0);
+      SearchTrace(tree,ply,depth,wtm,beta-1,beta,"Search",0);
 #endif
-    position[ply+1]=position[ply];
+    tree->position[ply+1]=tree->position[ply];
     Rule50Moves(ply+1)++;
     save_hash_key=HashKey;
     if (EnPassant(ply)) {
       HashEP(EnPassant(ply+1),HashKey);
       EnPassant(ply+1)=0;
     }
-    value=-ABSearch(-beta,-alpha,ChangeSide(wtm),
+    value=-ABSearch(tree,-beta,-beta+1,ChangeSide(wtm),
                     depth-NULL_MOVE_DEPTH-INCREMENT_PLY,ply+1,NO_NULL);
     HashKey=save_hash_key;
-    if (abort_search) return(0);
+    if (abort_search || tree->stop) return(0);
     if (value >= beta) {
-      StoreRefutation(ply,depth,wtm,value,threat);
+      StoreRefutation(tree,ply,depth,wtm,value,threat);
       return(value);
     }
     if (value < -MATE+30) threat=1;
@@ -240,24 +247,24 @@ int Search(int alpha, int beta, int wtm, int depth, int ply, int do_null)
 |                                                          |
  ----------------------------------------------------------
 */
-  next_status[ply].phase=FIRST_PHASE;
-  if (hash_move[ply]==0 && (depth > 2*INCREMENT_PLY) &&
+  tree->next_status[ply].phase=FIRST_PHASE;
+  if (tree->hash_move[ply]==0 && (depth > 2*INCREMENT_PLY) &&
       (((ply & 1) && alpha == root_alpha && beta == root_beta) ||
       (!(ply & 1) && alpha == -root_beta && beta == -root_alpha))) {
-    current_move[ply]=0;
-    value=ABSearch(alpha,beta,wtm,depth-2*INCREMENT_PLY,ply,DO_NULL);
-    if (abort_search) return(0);
+    tree->current_move[ply]=0;
+    value=ABSearch(tree,alpha,beta,wtm,depth-2*INCREMENT_PLY,ply,DO_NULL);
+    if (abort_search || tree->stop) return(0);
     if (value <= alpha) {
-      value=ABSearch(-MATE,beta,wtm,depth-2*INCREMENT_PLY,ply,DO_NULL);
-      if (abort_search) return(0);
+      value=ABSearch(tree,-MATE,beta,wtm,depth-2*INCREMENT_PLY,ply,DO_NULL);
+      if (abort_search || tree->stop) return(0);
     }
     else if (value < beta) {
-      if ((int) pv[ply-1].path_length >= ply) 
-        hash_move[ply]=pv[ply-1].path[ply];
+      if ((int) tree->pv[ply-1].path_length >= ply) 
+        tree->hash_move[ply]=tree->pv[ply-1].path[ply];
     }
-    else hash_move[ply]=current_move[ply];
-    last[ply]=last[ply-1];
-    next_status[ply].phase=FIRST_PHASE;
+    else tree->hash_move[ply]=tree->current_move[ply];
+    tree->last[ply]=tree->last[ply-1];
+    tree->next_status[ply].phase=FIRST_PHASE;
   }
 /*
  ----------------------------------------------------------
@@ -270,12 +277,12 @@ int Search(int alpha, int beta, int wtm, int depth, int ply, int do_null)
 |                                                          |
  ----------------------------------------------------------
 */
-  while ((current_phase[ply]=(in_check[ply]) ? NextEvasion(ply,wtm) : 
-                                               NextMove(ply,wtm))) {
-    extended_reason[ply]&=check_extension;
+  while ((tree->current_phase[ply]=(tree->in_check[ply]) ? NextEvasion(tree,ply,wtm) : 
+                                               NextMove(tree,ply,wtm))) {
+    tree->extended_reason[ply]&=check_extension;
 #if !defined(FAST)
     if (ply <= trace_level)
-      SearchTrace(ply,depth,wtm,alpha,beta,"Search",current_phase[ply]);
+      SearchTrace(tree,ply,depth,wtm,alpha,beta,"Search",tree->current_phase[ply]);
 #endif
 /*
  ----------------------------------------------------------
@@ -287,16 +294,18 @@ int Search(int alpha, int beta, int wtm, int depth, int ply, int do_null)
 |                                                          |
  ----------------------------------------------------------
 */
-    extensions=(threat) ? -15 : -60;
-    if (Captured(current_move[ply]) && Captured(current_move[ply-1]) &&
-        To(current_move[ply-1]) == To(current_move[ply]) &&
-        (p_values[Captured(current_move[ply-1])+7] == 
-         p_values[Captured(current_move[ply])+7] ||
-         Promote(current_move[ply-1])) &&
-        !(extended_reason[ply-1]&recapture_extension)) {
-      extended_reason[ply]|=recapture_extension;
-      recapture_extensions_done++;
-      extensions+=RECAPTURE;
+    extensions=-60;
+    if (threat) 
+      extensions+=(ply<=2*iteration_depth) ? threat_depth : threat_depth>>1;
+    if (Captured(tree->current_move[ply]) && Captured(tree->current_move[ply-1]) &&
+        To(tree->current_move[ply-1]) == To(tree->current_move[ply]) &&
+        (p_values[Captured(tree->current_move[ply-1])+7] == 
+         p_values[Captured(tree->current_move[ply])+7] ||
+         Promote(tree->current_move[ply-1])) &&
+        !(tree->extended_reason[ply-1]&recapture_extension)) {
+      tree->extended_reason[ply]|=recapture_extension;
+      tree->recapture_extensions_done++;
+      extensions+=(ply<=2*iteration_depth) ? recap_depth : recap_depth>>1;
     }
 /*
  ----------------------------------------------------------
@@ -306,17 +315,17 @@ int Search(int alpha, int beta, int wtm, int depth, int ply, int do_null)
 |                                                          |
  ----------------------------------------------------------
 */
-    if (Piece(current_move[ply])==pawn && 
-         ((wtm && To(current_move[ply])>H5 && TotalBlackPieces<16 &&
-          !And(mask_pawn_passed_w[To(current_move[ply])],BlackPawns)) ||
-         (!wtm && To(current_move[ply])<A4 && TotalWhitePieces<16 &&
-          !And(mask_pawn_passed_b[To(current_move[ply])],WhitePawns)) ||
-         push_extensions[To(current_move[ply])]) &&
-         Swap(From(current_move[ply]),To(current_move[ply]),wtm) ==
-           p_values[Captured(current_move[ply])+7]) {
-      extended_reason[ply]|=passed_pawn_extension;
-      passed_pawn_extensions_done++;
-      extensions+=PUSH_PASSED_PAWN;
+    if (Piece(tree->current_move[ply])==pawn && 
+         ((wtm && To(tree->current_move[ply])>H5 && TotalBlackPieces<16 &&
+          !And(mask_pawn_passed_w[To(tree->current_move[ply])],BlackPawns)) ||
+         (!wtm && To(tree->current_move[ply])<A4 && TotalWhitePieces<16 &&
+          !And(mask_pawn_passed_b[To(tree->current_move[ply])],WhitePawns)) ||
+         push_extensions[To(tree->current_move[ply])]) &&
+         Swap(tree,From(tree->current_move[ply]),To(tree->current_move[ply]),wtm) ==
+           p_values[Captured(tree->current_move[ply])+7]) {
+      tree->extended_reason[ply]|=passed_pawn_extension;
+      tree->passed_pawn_extensions_done++;
+      extensions+=(ply<=2*iteration_depth) ? pushpp_depth : pushpp_depth>>1;
     }
 /*
  ----------------------------------------------------------
@@ -329,8 +338,8 @@ int Search(int alpha, int beta, int wtm, int depth, int ply, int do_null)
 |                                                          |
  ----------------------------------------------------------
 */
-    MakeMove(ply,current_move[ply],wtm);
-    if (in_check[ply] || !Check(wtm)) {
+    MakeMove(tree,ply,tree->current_move[ply],wtm);
+    if (tree->in_check[ply] || !Check(wtm)) {
 /*
  ----------------------------------------------------------
 |                                                          |
@@ -341,14 +350,14 @@ int Search(int alpha, int beta, int wtm, int depth, int ply, int do_null)
  ----------------------------------------------------------
 */
       if (Check(ChangeSide(wtm))) {
-        in_check[ply+1]=1;
-        extended_reason[ply+1]=check_extension;
-        check_extensions_done++;
-        extensions+=IN_CHECK;
+        tree->in_check[ply+1]=1;
+        tree->extended_reason[ply+1]=check_extension;
+        tree->check_extensions_done++;
+        extensions+=(ply<=2*iteration_depth) ? incheck_depth : incheck_depth>>1;
       }
       else {
-        in_check[ply+1]=0;
-        extended_reason[ply+1]=no_extension;
+        tree->in_check[ply+1]=0;
+        tree->extended_reason[ply+1]=no_extension;
       }
 /*
  ----------------------------------------------------------
@@ -361,8 +370,8 @@ int Search(int alpha, int beta, int wtm, int depth, int ply, int do_null)
  ----------------------------------------------------------
 */
       if (depth<3*INCREMENT_PLY && depth>=2*INCREMENT_PLY &&
-          !in_check[ply] && extensions == -60) {
-        register int value=-Evaluate(ply+1,ChangeSide(wtm),
+          !tree->in_check[ply] && extensions == -60) {
+        register int value=-Evaluate(tree,ply+1,ChangeSide(wtm),
                                      -(beta+51),-(alpha-51));
         if (value+50 < alpha) extensions-=60;
       }
@@ -374,49 +383,79 @@ int Search(int alpha, int beta, int wtm, int depth, int ply, int do_null)
 |                                                          |
  ----------------------------------------------------------
 */
-      if (first_move) {
-        if (last[ply]-last[ply-1] == 1) {
-          extended_reason[ply]|=one_reply_extension;
-          one_reply_extensions_done++;
-          extensions+=ONE_REPLY_TO_CHECK;
+      if (!moves_searched) {
+        if (tree->last[ply]-tree->last[ply-1] == 1) {
+          tree->extended_reason[ply]|=one_reply_extension;
+          tree->one_reply_extensions_done++;
+          extensions+=(ply<=2*iteration_depth) ? onerep_depth : onerep_depth>>1;
         }
         extensions=Min(extensions,0);
-        value=-ABSearch(-beta,-alpha,ChangeSide(wtm),
+        value=-ABSearch(tree,-beta,-alpha,ChangeSide(wtm),
                         depth+extensions,ply+1,DO_NULL);
-        if (abort_search) {
-          UnMakeMove(ply,current_move[ply],wtm);
+        if (abort_search || tree->stop) {
+          UnMakeMove(tree,ply,tree->current_move[ply],wtm);
           return(0);
         }
-        first_move=0;
       }
       else {
         extensions=Min(extensions,0);
-        value=-ABSearch(-alpha-1,-alpha,ChangeSide(wtm),
+        value=-ABSearch(tree,-alpha-1,-alpha,ChangeSide(wtm),
                         depth+extensions,ply+1,DO_NULL);
-        if (abort_search) {
-          UnMakeMove(ply,current_move[ply],wtm);
+        if (abort_search || tree->stop) {
+          UnMakeMove(tree,ply,tree->current_move[ply],wtm);
           return(0);
         }
         if (value>alpha && value<beta) {
-          value=-ABSearch(-beta,-alpha,ChangeSide(wtm),
+          value=-ABSearch(tree,-beta,-alpha,ChangeSide(wtm),
                           depth+extensions,ply+1,DO_NULL);
-          if (abort_search) {
-            UnMakeMove(ply,current_move[ply],wtm);
+          if (abort_search || tree->stop) {
+            UnMakeMove(tree,ply,tree->current_move[ply],wtm);
             return(0);
           }
         }
       }
       if (value > alpha) {
         if(value >= beta) {
-          HistoryRefutation(ply,depth,wtm);
-          UnMakeMove(ply,current_move[ply],wtm);
-          StoreRefutation(ply,depth,wtm,value,threat);
+          History(tree,ply,depth,wtm,tree->current_move[ply]);
+          UnMakeMove(tree,ply,tree->current_move[ply],wtm);
+          StoreRefutation(tree,ply,depth,wtm,value,threat);
+          tree->fail_high++;
+          if (!moves_searched) tree->fail_high_first++;
           return(value);
         }
         alpha=value;
       }
+      moves_searched++;
+    } else tree->nodes_searched++;
+    UnMakeMove(tree,ply,tree->current_move[ply],wtm);
+#if defined(SMP)
+    if (smp_idle) {
+      if (moves_searched>0 && min_thread_depth<=depth && 
+        (!tree->in_check[ply] || tree->last[ply]-tree->last[ply-1]>1)) {
+        tree->alpha=alpha;
+        tree->beta=beta;
+        tree->value=value;
+        tree->wtm=wtm;
+        tree->ply=ply;
+        tree->depth=depth;
+        tree->threat=threat;
+        if(Thread(tree)) {
+          if (CheckInput()) Interrupt(ply);
+          value=tree->search_value;
+          if (value > alpha) {
+            if(value >= beta) {
+              History(tree,ply,depth,wtm,tree->current_move[ply]);
+              StoreRefutation(tree,ply,depth,wtm,value,threat);
+              tree->fail_high++;
+              return(value);
+            }
+            alpha=value;
+            break;
+          }
+        }
+      }
     }
-    UnMakeMove(ply,current_move[ply],wtm);
+#endif
   }
 /*
  ----------------------------------------------------------
@@ -427,11 +466,10 @@ int Search(int alpha, int beta, int wtm, int depth, int ply, int do_null)
 |                                                          |
  ----------------------------------------------------------
 */
-  if (first_move == 1) {
-    value=(Check(wtm)) ? -(MATE-ply) :
-                         ((wtm==root_wtm) ? DrawScore() : -DrawScore());
-    if (value >=alpha && value <beta) {
-      SavePV(ply,value,0);
+  if (moves_searched == 0) {
+    value=(Check(wtm)) ? -(MATE-ply) : DrawScore(root_wtm==wtm);
+    if (value>=alpha && value<beta) {
+      SavePV(tree,ply,value,0);
 #if !defined(FAST)
       if (ply <= trace_level) printf("Search() no moves!  ply=%d\n",ply);
 #endif
@@ -440,28 +478,12 @@ int Search(int alpha, int beta, int wtm, int depth, int ply, int do_null)
   }
   else {
     if (alpha != initial_alpha) {
-      memcpy(&pv[ply-1].path[ply],&pv[ply].path[ply],(pv[ply].path_length-ply+1)*4);
-      memcpy(&pv[ply-1].path_hashed,&pv[ply].path_hashed,3);
-      pv[ply-1].path[ply-1]=current_move[ply-1];
-      HistoryBest(ply,depth,wtm);
-    }
-    StoreBest(ply,depth,wtm,alpha,initial_alpha,threat);
-/*
- ----------------------------------------------------------
-|                                                          |
-|   if the 50-move rule is drawing close, then adjust the  |
-|   score to reflect the impending draw.                   |
-|                                                          |
- ----------------------------------------------------------
-*/
-    if (Rule50Moves(ply) >= 99) {
-      value=(wtm==root_wtm) ? DrawScore() : -DrawScore();
-      if (value < beta) SavePV(ply,value,0);
-#if !defined(FAST)
-      if(ply <= trace_level) printf("draw by 50-move rule detected, ply=%d.\n",ply);
-#endif
-      return(value);
+      memcpy(&tree->pv[ply-1].path[ply],&tree->pv[ply].path[ply],(tree->pv[ply].path_length-ply+1)*4);
+      memcpy(&tree->pv[ply-1].path_hashed,&tree->pv[ply].path_hashed,3);
+      tree->pv[ply-1].path[ply-1]=tree->current_move[ply-1];
+      History(tree,ply,depth,wtm,tree->pv[ply].path[ply]);
     }
+    StoreBest(tree,ply,depth,wtm,alpha,initial_alpha,threat);
     return(alpha);
   }
 }
