@@ -24,7 +24,7 @@
 *                                                                              *
 ********************************************************************************
 */
-int Swap(int source, int target, int wtm)
+int Swap(TREE *tree, int source, int target, int wtm)
 {
   register BITBOARD attacks;
   register int attacked_piece;
@@ -38,7 +38,7 @@ int Swap(int source, int target, int wtm)
 |                                                          |
  ----------------------------------------------------------
 */
-  attacks=AttacksTo(target);
+  attacks=AttacksTo(tree,target);
 /*
  ----------------------------------------------------------
 |                                                          |
@@ -62,7 +62,7 @@ int Swap(int source, int target, int wtm)
   attacked_piece=p_values[PieceOnSquare(source)+7];
   Clear(source,attacks);
   direction=directions[target][source];
-  if (direction) attacks=SwapXray(attacks,source,direction);
+  if (direction) attacks=SwapXray(tree,attacks,source,direction);
 /*
  ----------------------------------------------------------
 |                                                          |
@@ -118,7 +118,7 @@ int Swap(int source, int target, int wtm)
     attacked_piece=p_values[PieceOnSquare(square)+7];
     Clear(square,attacks);
     direction=directions[target][square];
-    if (direction) attacks=SwapXray(attacks,square,direction);
+    if (direction) attacks=SwapXray(tree,attacks,square,direction);
     next_capture++;
     sign=-sign;
     color=ChangeSide(color);
@@ -160,7 +160,7 @@ int Swap(int source, int target, int wtm)
 *                                                                              *
 ********************************************************************************
 */
-BITBOARD SwapXray(BITBOARD attacks, int from, int direction)
+BITBOARD SwapXray(TREE *tree, BITBOARD attacks, int from, int direction)
 {
   switch (direction) {
   case 1: 
