@@ -47,7 +47,7 @@ void TimeAdjust(int time_used, PLAYER player)
   }
 }
 
-/* last modified 10/12/97 */
+/* last modified 06/15/98 */
 /*
 ********************************************************************************
 *                                                                              *
@@ -67,6 +67,7 @@ int TimeCheck(int abort)
   int time_used;
   int value, last_value, searched;
   int *i, ndone;
+  TREE *tree=local[0];
 /*
  ----------------------------------------------------------
 |                                                          |
@@ -77,10 +78,11 @@ int TimeCheck(int abort)
 |                                                          |
  ----------------------------------------------------------
 */
-  if (last[0]==(last[1]-1) && !pondering && !booking && !annotate_mode) return(1);
+  if (tree->last[0]==(tree->last[1]-1) && !booking && !annotate_mode &&
+      !pondering && iteration_depth>4) return(1);
   ndone=0;
-  for (i=last[0];i<last[1];i++)
-    if (searched_this_root_move[i-last[0]]) ndone++;
+  for (i=tree->last[0];i<tree->last[1];i++)
+    if (tree->searched_this_root_move[i-tree->last[0]]) ndone++;
   if (ndone == 1) abort=1;
   if (iteration_depth <= 2) return(0);
 /*
@@ -93,7 +95,10 @@ int TimeCheck(int abort)
  ----------------------------------------------------------
 */
   time_used=(ReadClock(time_type)-start_time);
-  if (nodes_searched>noise_level && (display_options&32) && time_used>burp) {
+  if (tree->nodes_searched>noise_level && (display_options&32) && time_used>burp) {
+#if defined(SMP)
+    Lock(lock_io);
+#endif
 #if defined(MACOS)
     printf("               %2i   %s\n",iteration_depth,DisplayTime(time_used));
 #else
@@ -101,8 +106,11 @@ int TimeCheck(int abort)
 #endif
     burp=(time_used/1500)*1500+1500;
     fflush(stdout);
+#if defined(SMP)
+    UnLock(lock_io);
+#endif
   }
-  if (search_depth || pondering || analyze_mode) return(0);
+  if (pondering || analyze_mode) return(0);
   if (time_used > absolute_time_limit) return(1);
   if (easy_move && !search_time_limit) {
     if (time_limit>100 && time_used<time_limit/3) return (0);
@@ -123,8 +131,8 @@ int TimeCheck(int abort)
 */
   if ((root_value == root_alpha) && !search_failed_low) {
     searched=0;
-    for (i=last[0];i<last[1];i++)
-      if (searched_this_root_move[i-last[0]]) searched++;
+    for (i=tree->last[0];i<tree->last[1];i++)
+      if (tree->searched_this_root_move[i-tree->last[0]]) searched++;
     if (searched == 1) return(1);
   }
 /*
