@@ -3,7 +3,7 @@
 #include "chess.h"
 #include "data.h"
 
-/* last modified 02/19/97 */
+/* last modified 03/11/98 */
 /*
 ********************************************************************************
 *                                                                              *
@@ -12,10 +12,10 @@
 *                                                                              *
 ********************************************************************************
 */
-static BITBOARD bit_move;
-void UnMakeMove(int ply, int move, int wtm)
+void UnMakeMove(TREE *tree, int ply, int move, int wtm)
 {
   register int piece, from, to, captured, promote;
+  BITBOARD bit_move;
 /*
  ----------------------------------------------------------
 |                                                          |
@@ -24,8 +24,8 @@ void UnMakeMove(int ply, int move, int wtm)
 |                                                          |
  ----------------------------------------------------------
 */
-  HashKey=save_hash_key[ply];
-  PawnHashKey=save_pawn_hash_key[ply];
+  HashKey=tree->save_hash_key[ply];
+  PawnHashKey=tree->save_pawn_hash_key[ply];
 /*
  ----------------------------------------------------------
 |                                                          |
@@ -94,18 +94,21 @@ UnMakePieceMove:
         case knight:
           Clear(to,WhiteKnights);
           TotalWhitePieces-=knight_v;
+          WhiteMinors--;
           Material-=KNIGHT_VALUE;
           break;
         case bishop:
           Clear(to,WhiteBishops);
           Clear(to,BishopsQueens);
           TotalWhitePieces-=bishop_v;
+          WhiteMinors--;
           Material-=BISHOP_VALUE;
           break;
         case rook:
           Clear(to,WhiteRooks);
           Clear(to,RooksQueens);
           TotalWhitePieces-=rook_v;
+          WhiteMajors--;
           Material-=ROOK_VALUE;
           break;
         case queen:
@@ -113,6 +116,7 @@ UnMakePieceMove:
           Clear(to,BishopsQueens);
           Clear(to,RooksQueens);
           TotalWhitePieces-=queen_v;
+          WhiteMajors-=2;
           Material-=QUEEN_VALUE;
           break;
         }
@@ -154,18 +158,21 @@ UnMakePieceMove:
         case knight:
           Clear(to,BlackKnights);
           TotalBlackPieces-=knight_v;
+          BlackMinors--;
           Material+=KNIGHT_VALUE;
           break;
         case bishop:
           Clear(to,BlackBishops);
           Clear(to,BishopsQueens);
           TotalBlackPieces-=bishop_v;
+          BlackMinors--;
           Material+=BISHOP_VALUE;
           break;
         case rook:
           Clear(to,BlackRooks);
           Clear(to,RooksQueens);
           TotalBlackPieces-=rook_v;
+          BlackMajors--;
           Material+=ROOK_VALUE;
           break;
         case queen:
@@ -173,6 +180,7 @@ UnMakePieceMove:
           Clear(to,BishopsQueens);
           Clear(to,RooksQueens);
           TotalBlackPieces-=queen_v;
+          BlackMajors-=2;
           Material+=QUEEN_VALUE;
           break;
         }
@@ -358,6 +366,7 @@ UnMakePieceMove:
         Set(to,BlackPieces);
         PieceOnSquare(to)=-knight;
         TotalBlackPieces+=knight_v;
+        BlackMinors++;
         Material-=KNIGHT_VALUE;
       }
       else {
@@ -365,6 +374,7 @@ UnMakePieceMove:
         Set(to,WhitePieces);
         PieceOnSquare(to)=knight;
         TotalWhitePieces+=knight_v;
+        WhiteMinors++;
         Material+=KNIGHT_VALUE;
       }
     break;
@@ -382,6 +392,7 @@ UnMakePieceMove:
         Set(to,BlackPieces);
         PieceOnSquare(to)=-bishop;
         TotalBlackPieces+=bishop_v;
+        BlackMinors++;
         Material-=BISHOP_VALUE;
       }
       else {
@@ -389,6 +400,7 @@ UnMakePieceMove:
         Set(to,WhitePieces);
         PieceOnSquare(to)=bishop;
         TotalWhitePieces+=bishop_v;
+        WhiteMinors++;
         Material+=BISHOP_VALUE;
       }
     break;
@@ -406,6 +418,7 @@ UnMakePieceMove:
         Set(to,BlackPieces);
         PieceOnSquare(to)=-rook;
         TotalBlackPieces+=rook_v;
+        BlackMajors++;
         Material-=ROOK_VALUE;
       }
       else {
@@ -413,6 +426,7 @@ UnMakePieceMove:
         Set(to,WhitePieces);
         PieceOnSquare(to)=rook;
         TotalWhitePieces+=rook_v;
+        WhiteMajors++;
         Material+=ROOK_VALUE;
       }
     break;
@@ -431,6 +445,7 @@ UnMakePieceMove:
         Set(to,BlackPieces);
         PieceOnSquare(to)=-queen;
         TotalBlackPieces+=queen_v;
+        BlackMajors+=2;
         Material-=QUEEN_VALUE;
       }
       else {
@@ -438,6 +453,7 @@ UnMakePieceMove:
         Set(to,WhitePieces);
         PieceOnSquare(to)=queen;
         TotalWhitePieces+=queen_v;
+        WhiteMajors+=2;
         Material+=QUEEN_VALUE;
       }
       break;
@@ -453,11 +469,11 @@ UnMakePieceMove:
       Print(128,"piece=%d,from=%d,to=%d,captured=%d\n",
             piece,from,to,captured);
       Print(128,"ply=%d\n",ply);
-      if (log_file) DisplayChessBoard(log_file,search);
+      if (log_file) DisplayChessBoard(log_file,tree->pos);
     }
   }
 #if defined(DEBUG)
-  ValidatePosition(ply,move,"UnMakeMove(2)");
+  ValidatePosition(tree,ply,move,"UnMakeMove(2)");
 #endif
   return;
 }
