@@ -158,7 +158,6 @@ int CheckInput(void) {
 
   if (!xboard && !ics && !isatty(fileno(stdin))) return(0);
   if (strchr(cmd_buffer,'\n')) return(1);
-/*   DosSleep(20);*/
   KbdPeek (&keyinfo, 0);
   if (keyinfo.fbStatus & KBDTRF_FINAL_CHAR_IN) i = 1;
   else i = 0;
@@ -261,7 +260,7 @@ void DisplayBitBoard(BITBOARD board)
 *                                                                              *
 ********************************************************************************
 */
-void DisplayChessBoard(FILE *display_file, CHESS_POSITION pos)
+void DisplayChessBoard(FILE *display_file, POSITION pos)
 {
   int display_board[64];
   char display_string[] =
@@ -352,11 +351,10 @@ void DisplayPieceBoards(int *white, int *black)
 *                                                                              *
 ********************************************************************************
 */
-void DisplayPV(int level, int wtm, int time, int value, CHESS_PATH pv) {
-#define PrintOK() (nodes_searched>noise_level || value>(MATE-100))
+void DisplayPV(TREE *tree, int level, int wtm, int time, int value, PATH *pv) {
+#define PrintOK() (tree->nodes_searched>noise_level || value>(MATE-100))
   char buffer[512], *buffp, *bufftemp;
-  int i, j, dummy, t_move_number;
-  int type;
+  int i, t_move_number, type, j, dummy;
 /*
  ----------------------------------------------------------
 |                                                          |
@@ -369,11 +367,11 @@ void DisplayPV(int level, int wtm, int time, int value, CHESS_PATH pv) {
   if (display_options&64) sprintf(buffer," %d.",move_number);
   else buffer[0]=0;
   if ((display_options&64) && !wtm) sprintf(buffer+strlen(buffer)," ...");
-  for (i=1;i<=(int) pv.path_length;i++) {
+  for (i=1;i<=(int) pv->path_length;i++) {
     if ((display_options&64) && i>1 && wtm)
       sprintf(buffer+strlen(buffer)," %d.",t_move_number);
-    sprintf(buffer+strlen(buffer)," %s",OutputMove(&pv.path[i],i,wtm));
-    MakeMove(i,pv.path[i],wtm);
+    sprintf(buffer+strlen(buffer)," %s",OutputMove(tree,pv->path[i],i,wtm));
+    MakeMove(tree,i,pv->path[i],wtm);
     wtm=ChangeSide(wtm);
     if (wtm) t_move_number++;
   }
@@ -387,19 +385,19 @@ void DisplayPV(int level, int wtm, int time, int value, CHESS_PATH pv) {
 |                                                          |
  ----------------------------------------------------------
 */
-  if(pv.path_hashed == 1) {
-    for (i=pv.path_length+1;i<MAXPLY;i++) {
-      LookUp(i,0,wtm,&dummy,&dummy,&dummy);
-      if (hash_move[i] && ValidMove(i,wtm,hash_move[i])) {
-        pv.path[i]=hash_move[i];
+  if(pv->path_hashed == 1) {
+    for (i=pv->path_length+1;i<MAXPLY;i++) {
+      LookUp(tree,i,0,wtm,&dummy,&dummy,&dummy);
+      if (tree->hash_move[i] && LegalMove(tree,i,wtm,tree->hash_move[i])) {
+        pv->path[i]=tree->hash_move[i];
         for (j=1;j<i;j++) 
-          if (pv.path[i] == pv.path[j]) break;
+          if (pv->path[i] == pv->path[j]) break;
         if (j < i) break;
-        pv.path_length++;
+        pv->path_length++;
         if ((display_options&64) && wtm)
           sprintf(buffer+strlen(buffer)," %d.",t_move_number);
-        sprintf(buffer+strlen(buffer)," %s",OutputMove(&pv.path[i],i,wtm));
-        MakeMove(i,pv.path[i],wtm);
+        sprintf(buffer+strlen(buffer)," %s",OutputMove(tree,pv->path[i],i,wtm));
+        MakeMove(tree,i,pv->path[i],wtm);
       }
       else break;
       wtm=ChangeSide(wtm);
@@ -407,7 +405,7 @@ void DisplayPV(int level, int wtm, int time, int value, CHESS_PATH pv) {
     }
     sprintf(buffer+strlen(buffer)," <HT>");
   }
-  else if(pv.path_hashed == 2) 
+  else if(pv->path_hashed == 2) 
     sprintf(buffer+strlen(buffer)," <EGTB>");
   strcpy(whisper_text,buffer);
   if (PrintOK()) {
@@ -429,14 +427,23 @@ void DisplayPV(int level, int wtm, int time, int value, CHESS_PATH pv) {
       if (bufftemp) Print(type,"                                    ");
     } while(bufftemp);
     Whisper(level,iteration_depth,end_time-start_time,whisper_value,
-            nodes_searched,0,whisper_text);
+            tree->nodes_searched,0,whisper_text);
   }
-  for (i=pv.path_length;i>0;i--) {
+  for (i=pv->path_length;i>0;i--) {
     wtm=ChangeSide(wtm);
-    UnMakeMove(i,pv.path[i],wtm);
+    UnMakeMove(tree,i,pv->path[i],wtm);
   }
 }
 
+char* DisplaySQ(unsigned int sq)
+{
+  static char out[3];
+  out[0]=(From(sq) & 7)+'a';
+  out[1]=(From(sq) / 8)+'1';
+  out[2]=0;
+  return(out);
+}
+
 char* DisplayHHMM(unsigned int time)
 {
   static char out[10];
@@ -458,8 +465,7 @@ char* DisplayTime(unsigned int time)
   return(out);
 }
 
-char* DisplayTimeWhisper(unsigned int time)
-{
+char* DisplayTimeWhisper(unsigned int time) {
   static char out[10];
 
   if (time < 6000) sprintf(out,"%.2f",(float) time/100.0);
@@ -470,6 +476,38 @@ char* DisplayTimeWhisper(unsigned int time)
   return(out);
 }
 
+void DisplayTreeState(TREE *tree, int sply, int spos, int maxply) {
+  int left, i, *mvp, parallel=0;
+  char buf[1024];
+  buf[0]=0;
+  if (sply == 1) {
+    for (left=0,mvp=tree->last[0];mvp<tree->last[1];mvp++) 
+      if (!tree->searched_this_root_move[mvp-tree->last[0]]) left++;
+    sprintf(buf,"%d:%d/%d  ",1,left,tree->last[1]-tree->last[0]);
+  }
+  else {
+    for (i=0;i<spos-6;i++) sprintf(buf+strlen(buf)," ");
+    sprintf(buf+strlen(buf),"[p%2d] ",tree->thread_id);
+  }
+  for (i=Max(sply,2);i<=maxply;i++) {
+    left=0;
+    for (mvp=tree->last[i-1];mvp<tree->last[i];mvp++) 
+      if (*mvp) left++;
+    sprintf(buf+strlen(buf),"%d:%d/%d  ",i,left,tree->last[i]-tree->last[i-1]);
+    if (!(i%8)) sprintf(buf+strlen(buf),"\n");
+    if (tree->nprocs>1 && tree->ply==i) {
+      parallel=strlen(buf);
+      break;
+    }
+    if (sply > 1) break;
+  }
+  printf("%s\n",buf);
+  if (sply == 1 && tree->nprocs) {
+    for (i=0;i<max_threads;i++) if (tree->siblings[i])
+        DisplayTreeState(tree->siblings[i], tree->ply+1,parallel, maxply);
+  }
+}
+
 void Display64bitWord(BITBOARD word)
 {
   union doub {
@@ -537,6 +575,45 @@ void DisplayChessMove(char *title, int move)
          Promote(move));
 }
 
+/* last modified 02/17/98 */
+/*
+********************************************************************************
+*                                                                              *
+*   FormatPV() is used to display a PV during the search.  it will also note   *
+*   when the PV was terminated by a hash table hit.                            *
+*                                                                              *
+********************************************************************************
+*/
+char *FormatPV(TREE *tree, int wtm, PATH pv) {
+#define PrintOK() (tree->nodes_searched>noise_level || value>(MATE-100))
+  static char buffer[512];
+  int i, t_move_number;
+/*
+ ----------------------------------------------------------
+|                                                          |
+|   initialize.                                            |
+|                                                          |
+ ----------------------------------------------------------
+*/
+  t_move_number=move_number;
+  if (display_options&64) sprintf(buffer," %d.",move_number);
+  else buffer[0]=0;
+  if ((display_options&64) && !wtm) sprintf(buffer+strlen(buffer)," ...");
+  for (i=1;i<=(int) pv.path_length;i++) {
+    if ((display_options&64) && i>1 && wtm)
+      sprintf(buffer+strlen(buffer)," %d.",t_move_number);
+    sprintf(buffer+strlen(buffer)," %s",OutputMove(tree,pv.path[i],i,wtm));
+    MakeMove(tree,i,pv.path[i],wtm);
+    wtm=ChangeSide(wtm);
+    if (wtm) t_move_number++;
+  }
+  for (i=pv.path_length;i>0;i--) {
+    wtm=ChangeSide(wtm);
+    UnMakeMove(tree,i,pv.path[i],wtm);
+  }
+  return (buffer);
+}
+
 #if defined(MACOS)
 unsigned int ReadClock(TIME_TYPE type)
 {
@@ -546,27 +623,31 @@ unsigned int ReadClock(TIME_TYPE type)
 unsigned int ReadClock(TIME_TYPE type)
 {
 #if defined(UNIX) || defined(AMIGA)
-  static struct tms t;
-  static struct timeval timeval;
-  static struct timezone timezone;
+  struct tms t;
+  struct timeval timeval;
+  struct timezone timezone;
+  BITBOARD cputime=0;
 #endif
 
   switch (type) {
 #if defined(UNIX) || defined(AMIGA)
     case cpu:
       (void) times(&t);
+      cputime=t.tms_utime+t.tms_stime+t.tms_cutime+t.tms_cstime;
 #  if defined(CLK_TCK)
-      return((t.tms_utime+t.tms_stime)*100/CLK_TCK);
+      cputime=cputime*100/CLK_TCK;
+      return((unsigned int) cputime);
 #  else
       if (!clk_tck) clk_tck = sysconf(_SC_CLK_TCK);
-      return((t.tms_utime+t.tms_stime)*100/clk_tck);
+      cputime=cputime*100/clk_tck;
+      return((unsigned int) cputime);
 #  endif
     case elapsed:
       gettimeofday(&timeval, &timezone);
-      return(timeval.tv_sec*100+(timeval.tv_usec / 10000L));
+      return(timeval.tv_sec*100+(timeval.tv_usec/10000));
     default:
       gettimeofday(&timeval, &timezone);
-      return(timeval.tv_usec);
+      return(timeval.tv_sec*100+(timeval.tv_usec/10000));
 #endif
 #if defined(NT_i386) || defined(NT_AXP)
     case cpu:
@@ -586,6 +667,23 @@ unsigned int ReadClock(TIME_TYPE type)
 }
 #endif
 
+#if defined(SMP)
+/*
+********************************************************************************
+*                                                                              *
+*   FindBlockID() converts a thread block pointer into an ID that is easier to *
+*   understand when debugging.                                                 *
+*                                                                              *
+********************************************************************************
+*/
+int FindBlockID(TREE *block) {
+  int i;
+  for (i=0;i<MAX_BLOCKS+1;i++)
+    if (block == local[i]) return(i);
+  return(-1);
+}
+#endif
+
 /*
 ********************************************************************************
 *                                                                              *
@@ -688,15 +786,16 @@ int KingPawnSquare(int pawn, int king, int queen, int ptm)
 */
 void NewGame(int save) {
   char filename[64];
-  static int save_book_selection_width=5, save_book_random=2;
+  static int save_book_selection_width=5;
   static int save_whisper=0, save_kibitz=0, save_channel=0;
   static int save_resign=0, save_resign_count=0, save_draw_count=0;
   static int save_learning=0;
+  static int save_accept_draws=0;
+  TREE *tree=local[0];
 
   new_game=0;
   if (save) {
     save_book_selection_width=book_selection_width;
-    save_book_random=book_random;
     save_whisper=whisper;
     save_kibitz=kibitz;
     save_channel=channel;
@@ -704,12 +803,20 @@ void NewGame(int save) {
     save_resign_count=resign_count;
     save_draw_count=draw_count;
     save_learning=learning;
+    save_accept_draws=accept_draws;
   }
   else {
     if (learning&book_learning && moves_out_of_book)
-      LearnBook(crafty_is_white,last_search_value,0,0,1);
+      LearnBook(tree,crafty_is_white,last_search_value,0,0,1);
     if (ics) printf("*whisper Hello from Crafty v%s !\n",version);
-    if (xboard) printf("kibitz Hello from Crafty v%s !\n",version);
+    if (xboard) {
+#if defined(SMP)
+      printf("kibitz Hello from Crafty v%s! (%d cpus)\n",
+             version,Max(1,max_threads));
+#else
+      printf("kibitz Hello from Crafty v%s!\n",version);
+#endif
+    }
     over=0;
     moves_out_of_book=0;
     ponder_move=0;
@@ -717,7 +824,7 @@ void NewGame(int save) {
     last_pv.path_iteration_depth=0;
     last_pv.path_length=0;
     strcpy(initial_position,"");
-    InitializeChessBoard(&position[0]);
+    InitializeChessBoard(&tree->position[0]);
     InitializeHashTables();
     force=0;
     no_tricks=1;
@@ -746,7 +853,6 @@ void NewGame(int save) {
 #endif
     history_file=fopen(filename,"w+");
     book_selection_width=save_book_selection_width;
-    book_random=save_book_random;
     whisper=save_whisper;
     kibitz=save_kibitz;
     channel=save_channel;
@@ -754,14 +860,17 @@ void NewGame(int save) {
     resign_count=save_resign_count;
     resign_counter=0;
     draw_count=save_draw_count;
+    accept_draws=save_accept_draws;
     draw_counter=0;
     usage_level=0;
     learning=save_learning;
-    largest_positional_score=60;
+    largest_positional_score=100;
     predicted=0;
     whisper_depth=0;
     whisper_value=0;
-    nodes_searched=0;
+    tree->nodes_searched=0;
+    tree->fail_high=0;
+    tree->fail_high_first=0;
     cpu_percent=0;
     whisper_text[0]=0;
   }
@@ -817,6 +926,28 @@ int ParseTime(char* string)
   return(time*60+minutes);
 }
 
+void Pass(void) {
+  char buffer[128];
+  int halfmoves_done=2*(move_number-1)+(1-wtm);
+  int prev_pass=0;
+  /* Was previous move a pass? */
+  if (halfmoves_done>0) {
+    fseek(history_file,(halfmoves_done-1)*10,SEEK_SET);
+    if (fscanf(history_file,"%s",buffer)==0 || strcmp(buffer,"pass")==0) {
+      prev_pass=1;
+    }
+  }
+  if (prev_pass) {
+    if (wtm) move_number--;
+  } else {
+    fseek(history_file,halfmoves_done*10,SEEK_SET);
+    fprintf(history_file,"%9s\n","pass");
+    if (!wtm) move_number++;
+  }
+  wtm=ChangeSide(wtm);
+}
+
+
 /*
 ********************************************************************************
 *                                                                              *
@@ -827,7 +958,7 @@ int ParseTime(char* string)
 *                                                                              *
 ********************************************************************************
 */
-int PinnedOnKing(int wtm, int square)
+int PinnedOnKing(TREE *tree, int wtm, int square)
 {
   register int ray;
   if (wtm) {
@@ -927,7 +1058,7 @@ void Print(int vb, char *fmt, ...)
   va_start(ap,fmt);
   if (vb&display_options) vprintf(fmt, ap);
   fflush(stdout);
-  if (time_limit > 99 || tc_time_remaining > 6000) {
+  if (time_limit>99 || tc_time_remaining>6000 || vb==4095) {
     if (log_file) vfprintf(log_file, fmt, ap);
     if (log_file) fflush(log_file);
   }
@@ -1022,7 +1153,7 @@ int Read(int wait, char *buffer) {
 */
   else if (!wait) {
     if (CheckInput()) {
-      readdata=ReadRaw();
+      readdata=ReadInput();
       if (!strchr(cmd_buffer,'\n')) return(0);
       if (!readdata) return(-1);
     }
@@ -1031,11 +1162,11 @@ int Read(int wait, char *buffer) {
 /*
    case 3:  the buffer does not contain a complete line, but we
    were asked to wait until a complete command is entered.  So we
-   hang by doing a ReadRaw() and continue doing so until we get
+   hang by doing a ReadInput() and continue doing so until we get
    a N/L character in the buffer.  Then we parse and return.
 */
   else while (!strchr(cmd_buffer,'\n')) {
-    readdata=ReadRaw();
+    readdata=ReadInput();
     if (!readdata) return(-1);
   }
 
@@ -1092,12 +1223,12 @@ int ReadParse(char *buffer, char *args[], char *delims) {
 /*
 ********************************************************************************
 *                                                                              *
-*   ReadRaw() reads data from the input_stream, and buffers this into the      *
+*   ReadInput() reads data from the input_stream, and buffers this into the    *
 *   command_buffer for later processing.                                       *
 *                                                                              *
 ********************************************************************************
 */
-int ReadRaw(void) {
+int ReadInput(void) {
   char buffer[512], *end;
   int bytes;
 
@@ -1138,7 +1269,7 @@ int ReadRaw(void) {
 *                                                                              *
 ********************************************************************************
 */
-int ReadChessMove(FILE *input, int wtm, int one_move) {
+int ReadChessMove(TREE *tree, FILE *input, int wtm, int one_move) {
 
   static char text[128];
   char *tmove;
@@ -1155,7 +1286,7 @@ int ReadChessMove(FILE *input, int wtm, int one_move) {
          (tmove[0]>='A' && tmove[0]<='Z')) ||
         !strcmp(tmove,"0-0") || !strcmp(tmove,"0-0-0")) {
       if (!strcmp(tmove,"exit")) return(-1);
-      move=InputMove(tmove,0,wtm,1,0);
+      move=InputMove(tree,tmove,0,wtm,1,0);
     }
     if (one_move) break;
   }
@@ -1172,7 +1303,7 @@ int ReadChessMove(FILE *input, int wtm, int one_move) {
 *                                                                              *
 ********************************************************************************
 */
-int ReadNextMove(char *text, int ply, int wtm) {
+int ReadNextMove(TREE *tree, char *text, int ply, int wtm) {
 
   char *tmove;
   int move=0;
@@ -1185,12 +1316,12 @@ int ReadNextMove(char *text, int ply, int wtm) {
        (tmove[0]>='A' && tmove[0]<='Z')) ||
       !strcmp(tmove,"0-0") || !strcmp(tmove,"0-0-0")) {
     if (!strcmp(tmove,"exit")) return(-1);
-    move=InputMove(tmove,ply,wtm,1,0);
+    move=InputMove(tree,tmove,ply,wtm,1,0);
   }
   return(move);
 }
 
-/* last modified 01/14/98 */
+/* last modified 06/15/98 */
 /*
 ********************************************************************************
 *                                                                              *
@@ -1205,7 +1336,7 @@ int ReadNextMove(char *text, int ply, int wtm) {
 int ReadPGN(FILE *input, int option) {
   static int data=0, lines_read=0;
   static char input_buffer[512];
-  char temp[512], *eof, analysis_move[16];
+  char temp[512], *eof, analysis_move[64];
   int braces=0, parens=0, brackets=0, analysis=0, last_good_line;
   
 /*
@@ -1217,6 +1348,7 @@ int ReadPGN(FILE *input, int option) {
 |                                                          |
  ----------------------------------------------------------
 */
+  pgn_suggested_percent=0;
   if (!input) {
     lines_read=0;
     data=0;
@@ -1237,7 +1369,7 @@ int ReadPGN(FILE *input, int option) {
       eof=fgets(input_buffer,512,input);
       if (!eof) return(-1);
       if (strchr(input_buffer,'\n')) *strchr(input_buffer,'\n')=0;
-      if (strchr(input_buffer,'\r')) *strchr(input_buffer,'\r')=0;
+      if (strchr(input_buffer,'\r')) *strchr(input_buffer,'\r')=' ';
       lines_read++;
       buffer[0]=0;
       sscanf(input_buffer,"%s",buffer);
@@ -1259,6 +1391,11 @@ int ReadPGN(FILE *input, int option) {
         else if (strstr(input_buffer,"BlackElo")) strcpy(pgn_black_elo,value);
         else if (strstr(input_buffer,"Black")) strcpy(pgn_black,value);
         else if (strstr(input_buffer,"Result")) strcpy(pgn_result,value);
+        else if (strstr(input_buffer,"FEN")) {
+          sprintf(buffer,"setboard %s",value);
+          (void) Option(local[0]);
+          continue;
+        }
         return(1);
       } while(0);
       data=1;
@@ -1299,38 +1436,38 @@ int ReadPGN(FILE *input, int option) {
       last_good_line=lines_read;
       analysis_move[0]=0;
       if (strchr(buffer,'{') || strchr(buffer,'(')) while (1) {
-        char *skip;
+        char *skip, *ch;
         analysis=1;
-        while (strpbrk(buffer,"(){}[]")) {
-          if (strchr(buffer,'(')) {
+        while ((ch=strpbrk(buffer,"(){}[]"))) {
+          if (*ch == '(') {
             *strchr(buffer,'(')=' ';
-            parens++;
+            if (!braces) parens++;
           }
-          if (strchr(buffer,')')) {
+          if (*ch == ')') {
             *strchr(buffer,')')=' ';
-            parens--;
+            if (!braces) parens--;
           }
-          if (strchr(buffer,'{')) {
+          if (*ch == '{') {
             *strchr(buffer,'{')=' ';
             braces++;
           }
-          if (strchr(buffer,'}')) {
+          if (*ch == '}') {
             *strchr(buffer,'}')=' ';
             braces--;
           }
-          if (strchr(buffer,'[')) {
+          if (*ch == '[') {
             *strchr(buffer,'[')=' ';
-            brackets++;
+            if (!braces) brackets++;
           }
-          if (strchr(buffer,']')) {
+          if (*ch == ']') {
             *strchr(buffer,']')=' ';
-            brackets--;
+            if (!braces) brackets--;
           }
         }
         if (analysis && analysis_move[0]==0) {
           if (strspn(buffer," ") != strlen(buffer)) {
             char *tmove=analysis_move;
-            sscanf(buffer,"%s",analysis_move);
+            sscanf(buffer,"%64s",analysis_move);
             strcpy(buffer,analysis_move);
             if (strcmp(buffer,"0-0") && strcmp(buffer,"0-0-0"))
               tmove=buffer+strspn(buffer,"0123456789.");
@@ -1344,7 +1481,7 @@ int ReadPGN(FILE *input, int option) {
               analysis_move[0]=0;
           }
         }
-        if (parens+braces+brackets == 0) break;
+        if (parens==0 && braces==0 && brackets==0) break;
         buffer[0]=0;
         sscanf(input_buffer,"%s",buffer);
         if (strlen(buffer) == 0) {
@@ -1356,7 +1493,7 @@ int ReadPGN(FILE *input, int option) {
             return(-1);
           }
           if (strchr(input_buffer,'\n')) *strchr(input_buffer,'\n')=0;
-          if (strchr(input_buffer,'\r')) *strchr(input_buffer,'\r')=0;
+          if (strchr(input_buffer,'\r')) *strchr(input_buffer,'\r')=' ';
           lines_read++;
           if (lines_read-last_good_line >= 100) {
             parens=0;
@@ -1378,7 +1515,17 @@ int ReadPGN(FILE *input, int option) {
           strcpy(temp,buffer+skip);
           strcpy(buffer,temp);
         }
-        if (isalpha(buffer[0]) || strchr(buffer,'-')) return(0);
+        if (isalpha(buffer[0]) || strchr(buffer,'-')) {
+          char *first, *last, *percent;
+          first=input_buffer+strspn(input_buffer," ");
+          if (first==0 || *first != '{') return(0);
+          last=strchr(input_buffer,'}');
+          if (last == 0) return(0);
+          percent=strstr(first,"play");
+          if (percent == 0) return(0);
+          pgn_suggested_percent=atoi(percent+4+strspn(percent+4," "));
+          return(0);
+        }
       }
       if (analysis_move[0] && option==1) {
         strcpy(buffer,analysis_move);
@@ -1389,8 +1536,36 @@ int ReadPGN(FILE *input, int option) {
   return(-1);
 }
 
-char* Reverse(void)
-{
+/* last modified 06/10/98 */
+/*
+********************************************************************************
+*                                                                              *
+*   RestoreGame() resets the position to the beginning of the game, and then   *
+*   reads in the game.nnn history file to set the position up so that the game *
+*   position matches the position at the end of the history file.              *
+*                                                                              *
+********************************************************************************
+*/
+void RestoreGame(void) {
+  int i, move;
+  char cmd[16];
+  wtm=1;
+  InitializeChessBoard(&local[0]->position[0]);
+  for (i=0;i<500;i++) {
+    fseek(history_file,i*10,SEEK_SET);
+    strcpy(cmd,"");
+    fscanf(history_file,"%s",cmd);
+    if (strcmp(cmd,"pass")) {
+      move=InputMove(local[0],cmd,0,wtm,1,0);
+      if (move) MakeMoveRoot(local[0],move,wtm);
+      else break;
+    }
+    wtm=ChangeSide(wtm);
+  } 
+  Phase();
+}
+
+char* Reverse(void) {
 #if defined(NT_i386) || defined(NT_AXP)
   HANDLE  std_console;
   std_console = GetStdHandle(STD_OUTPUT_HANDLE);
@@ -1413,22 +1588,22 @@ char* Reverse(void)
 
 #if !defined(USE_ASSEMBLY_A)
 
-BITBOARD AttacksDiaga1Func (DIAG_INFO *diag, CHESS_POSITION *boardp)
+BITBOARD AttacksDiaga1Func (DIAG_INFO *diag, POSITION *boardp)
 {
   return AttacksDiaga1Int(diag,boardp);
 }
 
-BITBOARD AttacksDiagh1Func(DIAG_INFO *diag, CHESS_POSITION *boardp)
+BITBOARD AttacksDiagh1Func(DIAG_INFO *diag, POSITION *boardp)
 {
   return AttacksDiagh1Int(diag,boardp);
 }
 
-BITBOARD AttacksFileFunc(int square, CHESS_POSITION *boardp)
+BITBOARD AttacksFileFunc(int square, POSITION *boardp)
 {
   return AttacksFileInt(square,boardp);
 }
 
-BITBOARD AttacksRankFunc(int square, CHESS_POSITION *boardp)
+BITBOARD AttacksRankFunc(int square, POSITION *boardp)
 {
   BITBOARD tmp = Or((boardp)->w_occupied, (boardp)->b_occupied);
 
@@ -1440,13 +1615,13 @@ BITBOARD AttacksRankFunc(int square, CHESS_POSITION *boardp)
   return SplitShiftl (tmp2, Rank(~(square))<<3);
 }
 
-BITBOARD AttacksBishopFunc(DIAG_INFO *diag, CHESS_POSITION *boardp)
+BITBOARD AttacksBishopFunc(DIAG_INFO *diag, POSITION *boardp)
 {
   return Or(AttacksDiaga1Int(diag,boardp),
       AttacksDiagh1Int(diag,boardp));
 }
 
-BITBOARD AttacksRookFunc(int square, CHESS_POSITION *boardp)
+BITBOARD AttacksRookFunc(int square, POSITION *boardp)
 {
   BITBOARD tmp = Or((boardp)->w_occupied, (boardp)->b_occupied);
 
@@ -1458,22 +1633,22 @@ BITBOARD AttacksRookFunc(int square, CHESS_POSITION *boardp)
       AttacksFileInt(square,boardp));
 }
 
-unsigned MobilityDiaga1Func(DIAG_INFO *diag, CHESS_POSITION *boardp)
+unsigned MobilityDiaga1Func(DIAG_INFO *diag, POSITION *boardp)
 {
   return MobilityDiaga1Int(diag,boardp);
 }
 
-unsigned MobilityDiagh1Func(DIAG_INFO *diag, CHESS_POSITION *boardp)
+unsigned MobilityDiagh1Func(DIAG_INFO *diag, POSITION *boardp)
 {
   return MobilityDiagh1Int(diag,boardp);
 }
 
-unsigned MobilityFileFunc(int square, CHESS_POSITION *boardp)
+unsigned MobilityFileFunc(int square, POSITION *boardp)
 {
   return MobilityFileInt (square,boardp);
 }
 
-unsigned MobilityRankFunc(int square, CHESS_POSITION *boardp)
+unsigned MobilityRankFunc(int square, POSITION *boardp)
 {
   return MobilityRankInt (square,boardp);
 }
@@ -1826,11 +2001,11 @@ void Whisper(int level,int depth,int time,int value,unsigned int nodes,
                (time)?100*nodes/time:nodes,DisplayTimeWhisper(time),cpu,predicted);
       }
     case 3:
-      if ((kibitz >= 3) && nodes>5000) {
+      if ((kibitz >= 3) && (nodes>5000 || level==2)) {
         if (ics) printf("*");
         printf("kibitz pv:%s\n",pv);
       }
-      else if ((whisper >= 3) && nodes>5000) {
+      else if ((whisper >= 3) && (nodes>5000 || level==2)) {
         if (ics) printf("*");
         printf("%s pv:%s\n",prefix,pv);
       }
@@ -1878,7 +2053,7 @@ void Whisper(int level,int depth,int time,int value,unsigned int nodes,
       }
       break;
     }
-    if (post) {
+    if (post && level>1) {
       if (strstr(pv,"book"))
         printf("	%2d  %5d %7d %6u %s\n",depth,value,time,nodes,pv+10);
       else
@@ -1887,3 +2062,146 @@ void Whisper(int level,int depth,int time,int value,unsigned int nodes,
     fflush(stdout);
   }
 }
+
+#if defined(SMP)
+
+/*
+********************************************************************************
+*                                                                              *
+*   CopyFromSMP() is used to copy data from a child thread to a parent thread. *
+*   this only copies the appropriate parts of the TREE structure to avoid      *
+*   burning memory bandwidth by copying everything.                            *
+*                                                                              *
+********************************************************************************
+*/
+void CopyFromSMP(TREE *p, TREE *c) {
+  int i;
+  if (c->nodes_searched && !c->stop && c->search_value > p->search_value) {
+    p->pv[p->ply]=c->pv[p->ply];
+    p->search_value=c->search_value;
+    for (i=1;i<MAXPLY;i++) {
+      p->killer_move1[i]=c->killer_move1[i];
+      p->killer_move2[i]=c->killer_move2[i];
+    }
+  }
+  p->nodes_searched+=c->nodes_searched;
+  p->fail_high+=c->fail_high;
+  p->fail_high_first+=c->fail_high_first;
+  p->evaluations+=c->evaluations;
+  p->transposition_probes+=c->transposition_probes;
+  p->transposition_hits+=c->transposition_hits;
+  p->pawn_probes+=c->pawn_probes;
+  p->pawn_hits+=c->pawn_hits;
+  p->tb_probes+=c->tb_probes;
+  p->tb_probes_successful+=c->tb_probes_successful;
+  p->check_extensions_done+=c->check_extensions_done;
+  p->recapture_extensions_done+=c->recapture_extensions_done;
+  p->passed_pawn_extensions_done+=c->passed_pawn_extensions_done;
+  p->one_reply_extensions_done+=c->one_reply_extensions_done;
+  c->used=0;
+}
+
+/*
+********************************************************************************
+*                                                                              *
+*   CopyToSMP() is used to copy data from a parent thread to a particular      *
+*   child thread.  this only copies the appropriate parts of the TREE          *
+*   structure to avoid burning memory bandwidth by copying everything.         *
+*                                                                              *
+********************************************************************************
+*/
+TREE* CopyToSMP(TREE *p) {
+  int i;
+  TREE *c;
+  for (i=1;i<MAX_BLOCKS+1 && local[i]->used;i++);
+  if (i > MAX_BLOCKS) {
+    Print(128, "ERROR.  no SMP block can be allocated\n");
+    return(0);
+  }
+  max_split_blocks=Max(max_split_blocks,i);
+  c=local[i];
+  c->used=1;
+  c->stop=0;
+  c->done=0;
+  for (i=0;i<max_threads;i++) c->siblings[i]=0;
+  c->pos=p->pos;
+  c->pv[p->ply-1]=p->pv[p->ply-1];
+  c->pv[p->ply]=p->pv[p->ply];
+  c->next_status[p->ply]=p->next_status[p->ply];
+  c->save_hash_key[p->ply]=p->save_hash_key[p->ply];
+  c->save_pawn_hash_key[p->ply]=p->save_pawn_hash_key[p->ply];
+  c->rephead_w=c->replist_w+(p->rephead_w-p->replist_w);
+  c->rephead_b=c->replist_b+(p->rephead_b-p->replist_b);
+  for (i=0;i<=p->rephead_w-p->replist_w+((p->ply-1)>>1);i++) 
+    c->replist_w[i]=p->replist_w[i];
+  for (i=0;i<=p->rephead_b-p->replist_b+((p->ply-1)>>1);i++) 
+    c->replist_b[i]=p->replist_b[i];
+  c->last[p->ply]=c->move_list;
+  c->hash_move[p->ply]=p->hash_move[p->ply];
+  for (i=1;i<=p->ply;i++) {
+    c->position[i]=p->position[i];
+    c->current_move[i]=p->current_move[i];
+    c->extended_reason[i]=p->extended_reason[i];
+    c->in_check[i]=p->in_check[i];
+    c->current_phase[i]=p->current_phase[i];
+  }
+  for (i=1;i<MAXPLY;i++) {
+    c->killer_move1[i]=p->killer_move1[i];
+    c->killer_move2[i]=p->killer_move2[i];
+  }
+  c->nodes_searched=0;
+  c->fail_high=0;
+  c->fail_high_first=0;
+  c->evaluations=0;
+  c->transposition_probes=0;
+  c->transposition_hits=0;
+  c->pawn_probes=0;
+  c->pawn_hits=0;
+  c->tb_probes=0;
+  c->tb_probes_successful=0;
+  c->check_extensions_done=0;
+  c->recapture_extensions_done=0;
+  c->passed_pawn_extensions_done=0;
+  c->one_reply_extensions_done=0;
+  c->alpha=p->alpha;
+  c->beta=p->beta;
+  c->value=p->value;
+  c->wtm=p->wtm;
+  c->ply=p->ply;
+  c->depth=p->depth;
+  c->threat=p->threat;
+  c->search_value=0;
+  return(c);
+}
+
+#endif
+
+/* last modified 07/07/98 */
+/*
+********************************************************************************
+*                                                                              *
+*   LegalMove() tests a move to confirm it is absolutely legal.  it should not *
+*   be used inside the search, but can be used to check a 21-bit (compressed)  *
+*   move to be sure it is safe to make it on the permanent game board.         *
+*                                                                              *
+********************************************************************************
+*/
+int LegalMove(TREE *tree, int ply, int wtm, int move) {
+  int moves[220], *mv, *mvp;
+/*
+   generate moves, then eliminate any that are illegal.               
+*/
+  if (move == 0) return(0);
+  tree->position[MAXPLY]=tree->position[ply];
+  mvp=GenerateCaptures(tree,MAXPLY, wtm, moves);
+  mvp=GenerateNonCaptures(tree,MAXPLY, wtm, mvp);
+  for (mv=&moves[0];mv<mvp;mv++) {
+    MakeMove(tree,MAXPLY, *mv, wtm);
+    if (!Check(wtm) && move==*mv) {
+      UnMakeMove(tree,MAXPLY, *mv, wtm);
+      return(1);
+    }
+    UnMakeMove(tree,MAXPLY, *mv, wtm);
+  }
+  return(0);
+}
